'''OpenGL extension EXT.rescale_normal

Overview (from the spec)
	
	When normal rescaling is enabled a new operation is added to the
	transformation of the normal vector into eye coordinates.  The normal vector 
	is rescaled after it is multiplied by the inverse modelview matrix and 
	before it is normalized.  
	
	The rescale factor is chosen so that in many cases normal vectors with unit
	length in object coordinates will not need to be normalized as they
	are transformed into eye coordinates.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/rescale_normal.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_rescale_normal'
GL_RESCALE_NORMAL_EXT = constant.Constant( 'GL_RESCALE_NORMAL_EXT', 0x803A )


def glInitRescaleNormalEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.vertex_shader

Overview (from the spec)
	
	This extension adds programmable vertex level processing to OpenGL. The
	application can write vertex shaders in a high level language as defined
	in the OpenGL Shading Language specification. The language itself is not
	discussed here. A vertex shader replaces the transformation, texture
	coordinate generation and lighting parts of OpenGL, and it also adds
	texture access at the vertex level. Furthermore, management of vertex
	shader objects and loading generic attributes are discussed. A vertex
	shader object, attached to a program object, can be compiled and linked
	to produce an executable that runs on the vertex processor in OpenGL.
	This extension also defines how such an executable interacts with the
	fixed functionality vertex processing of OpenGL 1.4.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ARB/vertex_shader.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_vertex_shader'
GL_VERTEX_SHADER_ARB = constant.Constant( 'GL_VERTEX_SHADER_ARB', 0x8B31 )
GL_MAX_VERTEX_UNIFORM_COMPONENTS_ARB = constant.Constant( 'GL_MAX_VERTEX_UNIFORM_COMPONENTS_ARB', 0x8B4A )
glget.addGLGetConstant( GL_MAX_VERTEX_UNIFORM_COMPONENTS_ARB, (1,) )
GL_MAX_VARYING_FLOATS_ARB = constant.Constant( 'GL_MAX_VARYING_FLOATS_ARB', 0x8B4B )
glget.addGLGetConstant( GL_MAX_VARYING_FLOATS_ARB, (1,) )
GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB = constant.Constant( 'GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB', 0x8B4C )
glget.addGLGetConstant( GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB, (1,) )
GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS_ARB = constant.Constant( 'GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS_ARB', 0x8B4D )
glget.addGLGetConstant( GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS_ARB, (1,) )
GL_OBJECT_ACTIVE_ATTRIBUTES_ARB = constant.Constant( 'GL_OBJECT_ACTIVE_ATTRIBUTES_ARB', 0x8B89 )
GL_OBJECT_ACTIVE_ATTRIBUTE_MAX_LENGTH_ARB = constant.Constant( 'GL_OBJECT_ACTIVE_ATTRIBUTE_MAX_LENGTH_ARB', 0x8B8A )
glBindAttribLocationARB = platform.createExtensionFunction( 
	'glBindAttribLocationARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhandleARB, constants.GLuint, arrays.GLcharARBArray,),
	doc = 'glBindAttribLocationARB( GLhandleARB(programObj), GLuint(index), GLcharARBArray(name) ) -> None',
	argNames = ('programObj', 'index', 'name',),
)

glGetActiveAttribARB = platform.createExtensionFunction( 
	'glGetActiveAttribARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhandleARB, constants.GLuint, constants.GLsizei, arrays.GLsizeiArray, arrays.GLintArray, arrays.GLuintArray, arrays.GLcharARBArray,),
	doc = 'glGetActiveAttribARB( GLhandleARB(programObj), GLuint(index), GLsizei(maxLength), GLsizeiArray(length), GLintArray(size), GLuintArray(type), GLcharARBArray(name) ) -> None',
	argNames = ('programObj', 'index', 'maxLength', 'length', 'size', 'type', 'name',),
)

glGetAttribLocationARB = platform.createExtensionFunction( 
	'glGetAttribLocationARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLint, 
	argTypes=(constants.GLhandleARB, arrays.GLcharARBArray,),
	doc = 'glGetAttribLocationARB( GLhandleARB(programObj), GLcharARBArray(name) ) -> constants.GLint',
	argNames = ('programObj', 'name',),
)


def glInitVertexShaderARB():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension NV.fragment_program2

Overview (from the spec)
	
	This extension, like the NV_fragment_program_option extension, provides
	additional fragment program functionality to extend the standard
	ARB_fragment_program language and execution environment.  ARB programs
	wishing to use this added functionality need only add:
	
	    OPTION NV_fragment_program2;
	
	to the beginning of their fragment programs.
	
	New functionality provided by this extension, above and beyond that
	already provided by the NV_fragment_program_option extension, includes:
	
	
	  * structured branching support, including data-dependent IF tests, loops
	    supporting a fixed number of iterations, and a data-dependent loop
	    exit instruction (BRK),
	
	  * subroutine calls,
	
	  * instructions to perform vector normalization, divide vector components
	    by a scalar, and perform two-component dot products (with or without a
	    scalar add),
	
	  * an instruction to perform a texture lookup with an explicit LOD,
	
	  * a loop index register for indirect access into the texture coordinate
	    attribute array, and
	
	  * a facing attribute that indicates whether the fragment is generated
	    from a front- or back-facing primitive.
	

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/NV/fragment_program2.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_fragment_program2'
GL_MAX_PROGRAM_EXEC_INSTRUCTIONS_NV = constant.Constant( 'GL_MAX_PROGRAM_EXEC_INSTRUCTIONS_NV', 0x88F4 )
GL_MAX_PROGRAM_CALL_DEPTH_NV = constant.Constant( 'GL_MAX_PROGRAM_CALL_DEPTH_NV', 0x88F5 )
GL_MAX_PROGRAM_IF_DEPTH_NV = constant.Constant( 'GL_MAX_PROGRAM_IF_DEPTH_NV', 0x88F6 )
GL_MAX_PROGRAM_LOOP_DEPTH_NV = constant.Constant( 'GL_MAX_PROGRAM_LOOP_DEPTH_NV', 0x88F7 )
GL_MAX_PROGRAM_LOOP_COUNT_NV = constant.Constant( 'GL_MAX_PROGRAM_LOOP_COUNT_NV', 0x88F8 )


def glInitFragmentProgram2NV():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""Image-handling routines

### Unresolved:

	Following methods are not yet resolved due to my not being sure how the 
	function should be wrapped:
	
		glCompressedTexImage3D
		glCompressedTexImage2D
		glCompressedTexImage1D
		glCompressedTexSubImage3D
		glCompressedTexSubImage2D
		glCompressedTexSubImage1D
"""
from OpenGL.raw import GL as simple
from OpenGL import images, arrays, wrapper, platform
import ctypes

def asInt( value ):
	if isinstance( value, float ):
		return int(round(value,0))
	return value

## update the image tables with standard image types...
#images.FORMAT_BITS.update( {
#	simple.GL_BITMAP : 1, # must be GL_UNSIGNED_BYTE
#	
#	simple.GL_RED : 8,
#	simple.GL_GREEN : 8,
#	simple.GL_BLUE : 8,
#	simple.GL_ALPHA : 8,
#	simple.GL_LUMINANCE : 8,
#	simple.GL_LUMINANCE_ALPHA : 8,
#	simple.GL_COLOR_INDEX : 8,
#	simple.GL_STENCIL_INDEX : 8,
#	simple.GL_DEPTH_COMPONENT : 8,
#	simple.GL_RGB : 24,
#	simple.GL_BGR : 24,
#
#	simple.GL_RGBA : 32,
#	simple.GL_BGRA : 32,
#	simple.GL_ABGR_EXT : 32,
#	simple.GL_CMYK_EXT : 32,
#
#	simple.GL_CMYKA_EXT : 40,
#	
#	simple.GL_YCRCB_422_SGIX : 8, # must be GL_UNSIGNED_BYTE
#	simple.GL_YCRCB_444_SGIX : 8, # must be GL_UNSIGNED_SHORT
#
#	simple.GL_FORMAT_SUBSAMPLE_24_24_OML : 32, # must be GL_UNSIGNED_INT_10_10_10_2
#	simple.GL_FORMAT_SUBSAMPLE_244_244_OML : 32, # must be GL_UNSIGNED_INT_10_10_10_2
#} )
images.COMPONENT_COUNTS.update( {
	simple.GL_BITMAP : 1, # must be GL_UNSIGNED_BYTE
	
	simple.GL_RED : 1,
	simple.GL_GREEN : 1,
	simple.GL_BLUE : 1,
	simple.GL_ALPHA : 1,
	simple.GL_LUMINANCE : 1,
	simple.GL_LUMINANCE_ALPHA : 2,
	simple.GL_COLOR_INDEX : 1,
	simple.GL_STENCIL_INDEX : 1,
	simple.GL_DEPTH_COMPONENT : 1,
	simple.GL_RGB : 3,
	simple.GL_BGR : 3,

	simple.GL_RGBA : 4,
	simple.GL_BGRA : 4,
	simple.GL_ABGR_EXT : 4,
	simple.GL_CMYK_EXT : 4,

	simple.GL_CMYKA_EXT : 5,
	
	simple.GL_YCRCB_422_SGIX : 1, # must be GL_UNSIGNED_BYTE
	simple.GL_YCRCB_444_SGIX : 1, # must be GL_UNSIGNED_SHORT

	simple.GL_FORMAT_SUBSAMPLE_24_24_OML : 1, # must be GL_UNSIGNED_INT_10_10_10_2
	simple.GL_FORMAT_SUBSAMPLE_244_244_OML : 1, # must be GL_UNSIGNED_INT_10_10_10_2
} )

#images.TYPE_TO_BITS.update( {
#	simple.GL_UNSIGNED_BYTE_3_3_2 : 8,
#	simple.GL_UNSIGNED_BYTE_2_3_3_REV : 8,
#	simple.GL_UNSIGNED_SHORT_4_4_4_4 : 16,
#	simple.GL_UNSIGNED_SHORT_4_4_4_4_REV : 16,
#	simple.GL_UNSIGNED_SHORT_5_5_5_1 : 16,
#	simple.GL_UNSIGNED_SHORT_1_5_5_5_REV : 16,
#	simple.GL_UNSIGNED_SHORT_5_6_5 : 16,
#	simple.GL_UNSIGNED_SHORT_5_6_5_REV : 16,
#	simple.GL_UNSIGNED_INT_8_8_8_8 : 32,
#	simple.GL_UNSIGNED_INT_8_8_8_8_REV : 32,
#	simple.GL_UNSIGNED_INT_10_10_10_2 : 32,
#	simple.GL_UNSIGNED_INT_2_10_10_10_REV : 32,
#	simple.GL_UNSIGNED_BYTE : ctypes.sizeof(simple.GLubyte) * 8,
#	simple.GL_BYTE: ctypes.sizeof(simple.GLbyte) * 8,
#	simple.GL_UNSIGNED_SHORT :  ctypes.sizeof(simple.GLushort) * 8,
#	simple.GL_SHORT :  ctypes.sizeof(simple.GLshort) * 8,
#	simple.GL_UNSIGNED_INT : ctypes.sizeof(simple.GLuint) * 8,
#	simple.GL_INT : ctypes.sizeof(simple.GLint) * 8,
#	simple.GL_FLOAT : ctypes.sizeof(simple.GLfloat) * 8,
#	simple.GL_DOUBLE : ctypes.sizeof(simple.GLdouble) * 8,
#} )
images.TYPE_TO_ARRAYTYPE.update( {
	simple.GL_UNSIGNED_BYTE_3_3_2 : simple.GL_UNSIGNED_BYTE,
	simple.GL_UNSIGNED_BYTE_2_3_3_REV : simple.GL_UNSIGNED_BYTE,
	simple.GL_UNSIGNED_SHORT_4_4_4_4 : simple.GL_UNSIGNED_SHORT,
	simple.GL_UNSIGNED_SHORT_4_4_4_4_REV : simple.GL_UNSIGNED_SHORT,
	simple.GL_UNSIGNED_SHORT_5_5_5_1 : simple.GL_UNSIGNED_SHORT,
	simple.GL_UNSIGNED_SHORT_1_5_5_5_REV : simple.GL_UNSIGNED_SHORT,
	simple.GL_UNSIGNED_SHORT_5_6_5 : simple.GL_UNSIGNED_SHORT,
	simple.GL_UNSIGNED_SHORT_5_6_5_REV : simple.GL_UNSIGNED_SHORT,
	simple.GL_UNSIGNED_INT_8_8_8_8 : simple.GL_UNSIGNED_INT,
	simple.GL_UNSIGNED_INT_8_8_8_8_REV : simple.GL_UNSIGNED_INT,
	simple.GL_UNSIGNED_INT_10_10_10_2 : simple.GL_UNSIGNED_INT,
	simple.GL_UNSIGNED_INT_2_10_10_10_REV : simple.GL_UNSIGNED_INT,
	simple.GL_UNSIGNED_BYTE : simple.GL_UNSIGNED_BYTE,
	simple.GL_BYTE: simple.GL_BYTE,
	simple.GL_UNSIGNED_SHORT : simple.GL_UNSIGNED_SHORT,
	simple.GL_SHORT :  simple.GL_SHORT,
	simple.GL_UNSIGNED_INT : simple.GL_UNSIGNED_INT,
	simple.GL_INT : simple.GL_INT,
	simple.GL_FLOAT : simple.GL_FLOAT,
	simple.GL_DOUBLE : simple.GL_DOUBLE,
} )
images.RANK_PACKINGS.update( {
	4: [
		(simple.glPixelStorei,simple.GL_PACK_SKIP_VOLUMES_SGIS, 0),
		(simple.glPixelStorei,simple.GL_PACK_IMAGE_DEPTH_SGIS, 0),
	],
	3: [
		(simple.glPixelStorei,simple.GL_PACK_SKIP_IMAGES, 0),
		(simple.glPixelStorei,simple.GL_PACK_IMAGE_HEIGHT, 0),
	],
	2: [
		(simple.glPixelStorei,simple.GL_PACK_ROW_LENGTH, 0),
		(simple.glPixelStorei,simple.GL_PACK_SKIP_ROWS, 0),
		(simple.glPixelStorei,simple.GL_PACK_ALIGNMENT, 1),
	],
	1: [
		(simple.glPixelStorei,simple.GL_PACK_SKIP_PIXELS, 0),
	],
} )


__all__ = (
	'glReadPixels',
	'glReadPixelsb',
	'glReadPixelsd',
	'glReadPixelsf',
	'glReadPixelsi',
	'glReadPixelss',
	'glReadPixelsub',
	'glReadPixelsui',
	'glReadPixelsus',
	
	'glGetTexImage',
	
	'glDrawPixels',
	'glDrawPixelsb',
	'glDrawPixelsf',
	'glDrawPixelsi',
	'glDrawPixelss',
	'glDrawPixelsub',
	'glDrawPixelsui',
	'glDrawPixelsus',
	
	
	'glTexSubImage2D',
	'glTexSubImage1D',
	#'glTexSubImage3D',
	
	'glTexImage1D',
	'glTexImage2D',
	#'glTexImage3D',
	
	'glGetTexImageb',
	'glGetTexImaged',
	'glGetTexImagef',
	'glGetTexImagei',
	'glGetTexImages',
	'glGetTexImageub',
	'glGetTexImageui',
	'glGetTexImageus',
	'glTexImage1Db',
	'glTexImage2Db',
	#'glTexImage3Db',
	'glTexSubImage1Db',
	'glTexSubImage2Db',
	#'glTexSubImage3Db',
	'glTexImage1Df',
	'glTexImage2Df',
	#'glTexImage3Df',
	'glTexSubImage1Df',
	'glTexSubImage2Df',
	#'glTexSubImage3Df',
	'glTexImage1Di',
	'glTexImage2Di',
	#'glTexImage3Di',
	'glTexSubImage1Di',
	'glTexSubImage2Di',
	#'glTexSubImage3Di',
	'glTexImage1Ds',
	'glTexImage2Ds',
	#'glTexImage3Ds',
	'glTexSubImage1Ds',
	'glTexSubImage2Ds',
	#'glTexSubImage3Ds',
	'glTexImage1Dub',
	'glTexImage2Dub',
	#'glTexImage3Dub',
	'glTexSubImage1Dub',
	'glTexSubImage2Dub',
	#'glTexSubImage3Dub',
	'glTexImage1Dui',
	'glTexImage2Dui',
	#'glTexImage3Dui',
	'glTexSubImage1Dui',
	'glTexSubImage2Dui',
	#'glTexSubImage3Dui',
	'glTexImage1Dus',
	'glTexImage2Dus',
	#'glTexImage3Dus',
	'glTexSubImage1Dus',
	'glTexSubImage2Dus',
	#'glTexSubImage3Dus',
	
	#'glColorTable',
	#'glGetColorTable',
	#'glColorSubTable',
	
	#'glConvolutionFilter1D',
	#'glConvolutionFilter2D',
	#'glGetConvolutionFilter',
	#'glSeparableFilter2D',
	#'glGetSeparableFilter',
	
	#'glGetMinmax',
)

for suffix,type in [
	('b',simple.GL_BYTE),
	('d',simple.GL_DOUBLE),
	('f',simple.GL_FLOAT),
	('i',simple.GL_INT),
	('s',simple.GL_SHORT),
	('ub',simple.GL_UNSIGNED_BYTE),
	('ui',simple.GL_UNSIGNED_INT),
	('us',simple.GL_UNSIGNED_SHORT),
]:
	def glReadPixels( x,y,width,height,format,type=type ):
		"""Read specified pixels from the current display buffer
		
		This typed version returns data in your specified default 
		array data-type format
		"""
		x,y,width,height = asInt(x),asInt(y),asInt(width),asInt(height)
		array = images.SetupPixelRead( format, (width,height), type )
		arrayType = arrays.GL_CONSTANT_TO_ARRAY_TYPE[ images.TYPE_TO_ARRAYTYPE.get(type,type) ]
		imageData = arrayType.dataPointer(array)
		simple.glReadPixels( 
			x,y,
			width, height,
			format,type, 
			ctypes.c_void_p( imageData )
		)
		return array
	globals()["glReadPixels%s"%(suffix,)] = glReadPixels
	def glGetTexImage( target, level,format,type=type ):
		"""Get a texture-level as an image"""
		from OpenGL.GL import glget
		dims = [glget.glGetTexLevelParameteriv( target, level, simple.GL_TEXTURE_WIDTH )]
		if target != simple.GL_TEXTURE_1D:
			dims.append( glget.glGetTexLevelParameteriv( target, level, simple.GL_TEXTURE_HEIGHT ) )
			if target != simple.GL_TEXTURE_2D:
				dims.append( glget.glGetTexLevelParameteriv( target, level, simple.GL_TEXTURE_DEPTH ) )
		array = images.SetupPixelRead( format, tuple(dims), type )
		arrayType = arrays.GL_CONSTANT_TO_ARRAY_TYPE[ images.TYPE_TO_ARRAYTYPE.get(type,type) ]
		simple.glGetTexImage( 
			target, level, format, type, ctypes.c_void_p( arrayType.dataPointer(array)) 
		)
		return array
	globals()["glGetTexImage%s"%(suffix,)] = glGetTexImage
##	def glGetTexSubImage( target, level,format,type ):
##		"""Get a texture-level as an image"""
##		from OpenGL.GL import glget
##		dims = [glget.glGetTexLevelParameteriv( target, level, simple.GL_TEXTURE_WIDTH )]
##		if target != simple.GL_TEXTURE_1D:
##			dims.append( glget.glGetTexLevelParameteriv( target, level, simple.GL_TEXTURE_HEIGHT ) )
##			if target != simple.GL_TEXTURE_2D:
##				dims.append( glget.glGetTexLevelParameteriv( target, level, simple.GL_TEXTURE_DEPTH ) )
##		array = images.SetupPixelRead( format, tuple(dims), type )
##		arrayType = arrays.GL_CONSTANT_TO_ARRAY_TYPE[ images.TYPE_TO_ARRAYTYPE.get(type,type) ]
##		simple.glGetTexImage( 
##			target, level, format, type, ctypes.c_void_p( arrayType.dataPointer(array)) 
##		)
##		return array
##	"%s = glGetTexImage"%(suffix)
	del suffix,type
# Now the real glReadPixels...
def glReadPixels( x,y,width,height,format,type, outputType=str ):
	"""Read specified pixels from the current display buffer
	
	x,y,width,height -- location and dimensions of the image to read 
		from the buffer
	format -- pixel format for the resulting data
	type -- data-format for the resulting data
	outputType -- default (str) provides string output of the 
		results iff OpenGL.UNSIGNED_BYTE_IMAGES_AS_STRING is True 
		and type == GL_UNSIGNED_BYTE.  Any other value will cause 
		output in the default array output format.
	
	returns the pixel data array in the format defined by the 
	format, type and outputType
	"""
	x,y,width,height = asInt(x),asInt(y),asInt(width),asInt(height)
	array = images.SetupPixelRead( format, (width,height), type )
	arrayType = arrays.GL_CONSTANT_TO_ARRAY_TYPE[ images.TYPE_TO_ARRAYTYPE.get(type,type) ]
	imageData = arrayType.dataPointer(array)
	simple.glReadPixels( 
		x,y,width,height,
		format,type, 
		ctypes.c_void_p( imageData ) 
	)
	if outputType is str:
		return images.returnFormat( array, type )
	else:
		return array

def glGetTexImage( target, level,format,type, outputType=str ):
	"""Get a texture-level as an image
	
	target -- enum constant for the texture engine to be read 
	level -- the mip-map level to read 
	format -- image format to read out the data 
	type -- data-type into which to read the data
	
	outputType -- default (str) provides string output of the 
		results iff OpenGL.UNSIGNED_BYTE_IMAGES_AS_STRING is True 
		and type == GL_UNSIGNED_BYTE.  Any other value will cause 
		output in the default array output format.
	
	returns the pixel data array in the format defined by the 
	format, type and outputType
	"""
	from OpenGL.GL import glget
	dims = [glget.glGetTexLevelParameteriv( target, level, simple.GL_TEXTURE_WIDTH )]
	if target != simple.GL_TEXTURE_1D:
		dims.append( glget.glGetTexLevelParameteriv( target, level, simple.GL_TEXTURE_HEIGHT ) )
		if target != simple.GL_TEXTURE_2D:
			dims.append( glget.glGetTexLevelParameteriv( target, level, simple.GL_TEXTURE_DEPTH ) )
	array = images.SetupPixelRead( format, tuple(dims), type )
	arrayType = arrays.GL_CONSTANT_TO_ARRAY_TYPE[ images.TYPE_TO_ARRAYTYPE.get(type,type) ]
	simple.glGetTexImage( 
		target, level, format, type, ctypes.c_void_p( arrayType.dataPointer(array)) 
	)
	if outputType is str:
		return images.returnFormat( array, type )
	else:
		return array


INT_DIMENSION_NAMES = [
	'width','height','depth','x','y','z',
	'xoffset','yoffset','zoffset',
	'start', 'count',
]
def asWrapper( value ):
	if not isinstance( value, wrapper.Wrapper ):
		return wrapper.wrapper( value )
	return value

def asIntConverter( value, *args ):
	if isinstance( value, float ):
		return int(round(value,0))
	return value

def setDimensionsAsInts( baseOperation ):
	"""Set arguments with names in INT_DIMENSION_NAMES to asInt processing"""
	baseOperation = asWrapper( baseOperation )
	argNames = getattr( baseOperation, 'pyConverterNames', baseOperation.argNames )
	for i,argName in enumerate(argNames):
		if argName in INT_DIMENSION_NAMES:
			baseOperation.setPyConverter( argName, asIntConverter )
	return baseOperation

	

class ImageInputConverter( object ):
	def __init__( self, rank, pixelsName=None, typeName='type' ):
		self.rank = rank
		self.typeName = typeName
		self.pixelsName = pixelsName
	def finalise( self, wrapper ):
		"""Get our pixel index from the wrapper"""
		self.typeIndex = wrapper.pyArgIndex( self.typeName )
		self.pixelsIndex = wrapper.pyArgIndex( self.pixelsName )
	def __call__( self, arg, baseOperation, pyArgs ):
		"""pyConverter for the pixels argument"""
		images.setupDefaultTransferMode()
		images.rankPacking( self.rank )
		type = pyArgs[ self.typeIndex ]
		try:
			arrayType = arrays.GL_CONSTANT_TO_ARRAY_TYPE[ images.TYPE_TO_ARRAYTYPE[ type ] ]
		except TypeError, err:
			import pdb
			pdb.set_trace()
		return arrayType.asArray( arg )
	def cResolver( self, array ):
		return ctypes.c_void_p( arrays.ArrayDatatype.dataPointer( array ) )
	
class TypedImageInputConverter( ImageInputConverter ):
	def __init__( self, rank, pixelsName, arrayType, typeName=None ):
		self.rank = rank
		self.arrayType = arrayType
		self.pixelsName = pixelsName
		self.typeName = typeName
	def __call__( self, arg, baseOperation, pyArgs ):
		"""The pyConverter for the pixels"""
		images.setupDefaultTransferMode()
		images.rankPacking( self.rank )
		return self.arrayType.asArray( arg )
	def finalise( self, wrapper ):
		"""Get our pixel index from the wrapper"""
		self.pixelsIndex = wrapper.pyArgIndex( self.pixelsName )
	def width( self, pyArgs, index, wrappedOperation ):
		"""Extract the width from the pixels argument"""
		return self.arrayType.dimensions( pyArgs[self.pixelsIndex] )[0]
	def height( self, pyArgs, index, wrappedOperation ):
		"""Extract the height from the pixels argument"""
		return self.arrayType.dimensions( pyArgs[self.pixelsIndex] )[1]
	def depth( self, pyArgs, index, wrappedOperation ):
		"""Extract the depth from the pixels argument"""
		return self.arrayType.dimensions( pyArgs[self.pixelsIndex] )[2]
	def type( self, pyArgs, index, wrappedOperation ):
		"""Provide the item-type argument from our stored value
		
		This is used for pre-bound processing where we want to provide 
		the type by implication...
		"""
		return self.typeName

class CompressedImageConverter( object ):
	def finalise( self, wrapper ):
		"""Get our pixel index from the wrapper"""
		self.dataIndex = wrapper.pyArgIndex( 'data' )
	def __call__( self, pyArgs, index, wrappedOperation ):
		"""Create a data-size measurement for our image"""
		arg = pyArgs[ self.dataIndex ]
		return arrays.ArrayType.arrayByteCount( arg )



DIMENSION_NAMES = (
	'width','height','depth'
)
PIXEL_NAMES = (
	'pixels', 'row', 'column',
)
DATA_SIZE_NAMES = (
	'imageSize',
)

def setImageInput( 
	baseOperation, arrayType=None, dimNames=DIMENSION_NAMES, 
	pixelName="pixels", typeName=None 
):
	"""Determine how to convert "pixels" into an image-compatible argument"""
	baseOperation = asWrapper( baseOperation )
	# rank is the count of width,height,depth arguments...
	rank = len([
		# rank is the number of dims we want, not the number we give...
		argName for argName in baseOperation.argNames 
		if argName in dimNames
	]) + 1
	if arrayType:
		converter = TypedImageInputConverter( rank, pixelName, arrayType, typeName=typeName )
		for i,argName in enumerate(baseOperation.argNames):
			if argName in dimNames:
				baseOperation.setPyConverter( argName )
				baseOperation.setCConverter( argName, getattr(converter,argName) )
			elif argName == 'type' and typeName is not None:
				baseOperation.setPyConverter( argName )
				baseOperation.setCConverter( argName, converter.type )
	else:
		converter = ImageInputConverter( rank, pixelsName=pixelName, typeName=typeName or 'type' )
	for argName in baseOperation.argNames:
		if argName in DATA_SIZE_NAMES:
			baseOperation.setPyConverter( argName )
			baseOperation.setCConverter( argName, converter.imageDataSize )
	baseOperation.setPyConverter(
		pixelName, converter,
	)
	baseOperation.setCResolver(
		pixelName, converter.cResolver
	)
	return baseOperation

glDrawPixels = setDimensionsAsInts(
	setImageInput(
		simple.glDrawPixels
	)
)
glTexSubImage2D = setDimensionsAsInts(
	setImageInput(
		simple.glTexSubImage2D
	)
)
glTexSubImage1D = setDimensionsAsInts(
	setImageInput(
		simple.glTexSubImage1D
	)
)
glTexImage2D = setDimensionsAsInts(
	setImageInput(
		simple.glTexImage2D
	)
)
glTexImage1D = setDimensionsAsInts(
	setImageInput(
		simple.glTexImage1D
	)
)

def typedImageFunction( suffix, arrayConstant,  baseFunction ):
	"""Produce a typed version of the given image function"""
	if baseFunction:
		arrayType = arrays.GL_CONSTANT_TO_ARRAY_TYPE[ arrayConstant ]
		function = setDimensionsAsInts(
			setImageInput(
				baseFunction, 
				arrayType,
				typeName = arrayConstant,
			)
		)
		functionName = baseFunction.__name__
		functionName = '%(functionName)s%(suffix)s'%locals()
		return functionName, function
	else:
		return baseFunction.__name__, baseFunction

def _setDataSize( baseFunction, argument='imageSize' ):
	"""Set the data-size value to come from the data field"""
	if baseFunction:
		converter = CompressedImageConverter()
		return asWrapper( baseFunction ).setPyConverter(
			argument
		).setCConverter( argument, converter )
	else:
		return baseFunction

def compressedImageFunction( baseFunction ):
	"""Set the imageSize and dimensions-as-ints converters for baseFunction"""
	if baseFunction:
		return setDimensionsAsInts(
			_setDataSize( 
				baseFunction, argument='imageSize'
			)
		)
	else:
		return baseFunction

for suffix,arrayConstant in [
	('b', simple.GL_BYTE),
	('f', simple.GL_FLOAT),
	('i', simple.GL_INT),
	('s', simple.GL_SHORT),
	('ub', simple.GL_UNSIGNED_BYTE),
	('ui', simple.GL_UNSIGNED_INT),
	('us', simple.GL_UNSIGNED_SHORT),
]:
	for functionName in (
		'glTexImage1D','glTexImage2D',
		'glTexSubImage1D','glTexSubImage2D',
		'glDrawPixels',
		#'glTexSubImage3D','glTexImage3D', # extension/1.2 standard
	):
		functionName, function = typedImageFunction(
			suffix, arrayConstant, getattr(simple,functionName),
		)
		globals()[functionName] = function
		del function, functionName
	del suffix,arrayConstant




'''OpenGL extension NV.vertex_program4

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/NV/vertex_program4.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_vertex_program4'
GL_VERTEX_ATTRIB_ARRAY_INTEGER_NV = constant.Constant( 'GL_VERTEX_ATTRIB_ARRAY_INTEGER_NV', 0x88FD )
glVertexAttribI1iEXT = platform.createExtensionFunction( 
	'glVertexAttribI1iEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLint,),
	doc = 'glVertexAttribI1iEXT( GLuint(index), GLint(x) ) -> None',
	argNames = ('index', 'x',),
)

glVertexAttribI2iEXT = platform.createExtensionFunction( 
	'glVertexAttribI2iEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLint, constants.GLint,),
	doc = 'glVertexAttribI2iEXT( GLuint(index), GLint(x), GLint(y) ) -> None',
	argNames = ('index', 'x', 'y',),
)

glVertexAttribI3iEXT = platform.createExtensionFunction( 
	'glVertexAttribI3iEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glVertexAttribI3iEXT( GLuint(index), GLint(x), GLint(y), GLint(z) ) -> None',
	argNames = ('index', 'x', 'y', 'z',),
)

glVertexAttribI4iEXT = platform.createExtensionFunction( 
	'glVertexAttribI4iEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLint, constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glVertexAttribI4iEXT( GLuint(index), GLint(x), GLint(y), GLint(z), GLint(w) ) -> None',
	argNames = ('index', 'x', 'y', 'z', 'w',),
)

glVertexAttribI1uiEXT = platform.createExtensionFunction( 
	'glVertexAttribI1uiEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint,),
	doc = 'glVertexAttribI1uiEXT( GLuint(index), GLuint(x) ) -> None',
	argNames = ('index', 'x',),
)

glVertexAttribI2uiEXT = platform.createExtensionFunction( 
	'glVertexAttribI2uiEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint, constants.GLuint,),
	doc = 'glVertexAttribI2uiEXT( GLuint(index), GLuint(x), GLuint(y) ) -> None',
	argNames = ('index', 'x', 'y',),
)

glVertexAttribI3uiEXT = platform.createExtensionFunction( 
	'glVertexAttribI3uiEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint, constants.GLuint, constants.GLuint,),
	doc = 'glVertexAttribI3uiEXT( GLuint(index), GLuint(x), GLuint(y), GLuint(z) ) -> None',
	argNames = ('index', 'x', 'y', 'z',),
)

glVertexAttribI4uiEXT = platform.createExtensionFunction( 
	'glVertexAttribI4uiEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint, constants.GLuint, constants.GLuint, constants.GLuint,),
	doc = 'glVertexAttribI4uiEXT( GLuint(index), GLuint(x), GLuint(y), GLuint(z), GLuint(w) ) -> None',
	argNames = ('index', 'x', 'y', 'z', 'w',),
)

glVertexAttribI1ivEXT = platform.createExtensionFunction( 
	'glVertexAttribI1ivEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLintArray,),
	doc = 'glVertexAttribI1ivEXT( GLuint(index), GLintArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribI2ivEXT = platform.createExtensionFunction( 
	'glVertexAttribI2ivEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLintArray,),
	doc = 'glVertexAttribI2ivEXT( GLuint(index), GLintArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribI3ivEXT = platform.createExtensionFunction( 
	'glVertexAttribI3ivEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLintArray,),
	doc = 'glVertexAttribI3ivEXT( GLuint(index), GLintArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribI4ivEXT = platform.createExtensionFunction( 
	'glVertexAttribI4ivEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLintArray,),
	doc = 'glVertexAttribI4ivEXT( GLuint(index), GLintArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribI1uivEXT = platform.createExtensionFunction( 
	'glVertexAttribI1uivEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLuintArray,),
	doc = 'glVertexAttribI1uivEXT( GLuint(index), GLuintArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribI2uivEXT = platform.createExtensionFunction( 
	'glVertexAttribI2uivEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLuintArray,),
	doc = 'glVertexAttribI2uivEXT( GLuint(index), GLuintArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribI3uivEXT = platform.createExtensionFunction( 
	'glVertexAttribI3uivEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLuintArray,),
	doc = 'glVertexAttribI3uivEXT( GLuint(index), GLuintArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribI4uivEXT = platform.createExtensionFunction( 
	'glVertexAttribI4uivEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLuintArray,),
	doc = 'glVertexAttribI4uivEXT( GLuint(index), GLuintArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribI4bvEXT = platform.createExtensionFunction( 
	'glVertexAttribI4bvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLbyteArray,),
	doc = 'glVertexAttribI4bvEXT( GLuint(index), GLbyteArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribI4svEXT = platform.createExtensionFunction( 
	'glVertexAttribI4svEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLshortArray,),
	doc = 'glVertexAttribI4svEXT( GLuint(index), GLshortArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribI4ubvEXT = platform.createExtensionFunction( 
	'glVertexAttribI4ubvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLubyteArray,),
	doc = 'glVertexAttribI4ubvEXT( GLuint(index), GLubyteArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribI4usvEXT = platform.createExtensionFunction( 
	'glVertexAttribI4usvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLushortArray,),
	doc = 'glVertexAttribI4usvEXT( GLuint(index), GLushortArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribIPointerEXT = platform.createExtensionFunction( 
	'glVertexAttribIPointerEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLint, constants.GLenum, constants.GLsizei, ctypes.c_void_p,),
	doc = 'glVertexAttribIPointerEXT( GLuint(index), GLint(size), GLenum(type), GLsizei(stride), c_void_p(pointer) ) -> None',
	argNames = ('index', 'size', 'type', 'stride', 'pointer',),
)

glGetVertexAttribIivEXT = platform.createExtensionFunction( 
	'glGetVertexAttribIivEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, arrays.GLintArray,),
	doc = 'glGetVertexAttribIivEXT( GLuint(index), GLenum(pname), GLintArray(params) ) -> None',
	argNames = ('index', 'pname', 'params',),
)

glGetVertexAttribIuivEXT = platform.createExtensionFunction( 
	'glGetVertexAttribIuivEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, arrays.GLuintArray,),
	doc = 'glGetVertexAttribIuivEXT( GLuint(index), GLenum(pname), GLuintArray(params) ) -> None',
	argNames = ('index', 'pname', 'params',),
)


def glInitVertexProgram4NV():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.framebuffer_blit

Overview (from the spec)
	
	This extension modifies EXT_framebuffer_object by splitting the
	framebuffer object binding point into separate DRAW and READ
	bindings.  This allows copying directly from one framebuffer to
	another.  In addition, a new high performance blit function is
	added to facilitate these blits and perform some data conversion
	where allowed.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/framebuffer_blit.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_framebuffer_blit'
GL_READ_FRAMEBUFFER_EXT = constant.Constant( 'GL_READ_FRAMEBUFFER_EXT', 0x8CA8 )
GL_DRAW_FRAMEBUFFER_EXT = constant.Constant( 'GL_DRAW_FRAMEBUFFER_EXT', 0x8CA9 )
GL_READ_FRAMEBUFFER_BINDING_EXT = constant.Constant( 'GL_READ_FRAMEBUFFER_BINDING_EXT', 0x8CAA )
glBlitFramebufferEXT = platform.createExtensionFunction( 
	'glBlitFramebufferEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLint, constants.GLint, constants.GLint, constants.GLint, constants.GLint, constants.GLint, constants.GLint, constants.GLbitfield, constants.GLenum,),
	doc = 'glBlitFramebufferEXT( GLint(srcX0), GLint(srcY0), GLint(srcX1), GLint(srcY1), GLint(dstX0), GLint(dstY0), GLint(dstX1), GLint(dstY1), GLbitfield(mask), GLenum(filter) ) -> None',
	argNames = ('srcX0', 'srcY0', 'srcX1', 'srcY1', 'dstX0', 'dstY0', 'dstX1', 'dstY1', 'mask', 'filter',),
)


def glInitFramebufferBlitEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.bindable_uniform

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/bindable_uniform.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_bindable_uniform'
GL_MAX_VERTEX_BINDABLE_UNIFORMS_EXT = constant.Constant( 'GL_MAX_VERTEX_BINDABLE_UNIFORMS_EXT', 0x8DE2 )
GL_MAX_FRAGMENT_BINDABLE_UNIFORMS_EXT = constant.Constant( 'GL_MAX_FRAGMENT_BINDABLE_UNIFORMS_EXT', 0x8DE3 )
GL_MAX_GEOMETRY_BINDABLE_UNIFORMS_EXT = constant.Constant( 'GL_MAX_GEOMETRY_BINDABLE_UNIFORMS_EXT', 0x8DE4 )
GL_MAX_BINDABLE_UNIFORM_SIZE_EXT = constant.Constant( 'GL_MAX_BINDABLE_UNIFORM_SIZE_EXT', 0x8DED )
GL_UNIFORM_BUFFER_EXT = constant.Constant( 'GL_UNIFORM_BUFFER_EXT', 0x8DEE )
GL_UNIFORM_BUFFER_BINDING_EXT = constant.Constant( 'GL_UNIFORM_BUFFER_BINDING_EXT', 0x8DEF )
glUniformBufferEXT = platform.createExtensionFunction( 
	'glUniformBufferEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLint, constants.GLuint,),
	doc = 'glUniformBufferEXT( GLuint(program), GLint(location), GLuint(buffer) ) -> None',
	argNames = ('program', 'location', 'buffer',),
)

glGetUniformBufferSizeEXT = platform.createExtensionFunction( 
	'glGetUniformBufferSizeEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLint, 
	argTypes=(constants.GLuint, constants.GLint,),
	doc = 'glGetUniformBufferSizeEXT( GLuint(program), GLint(location) ) -> constants.GLint',
	argNames = ('program', 'location',),
)

glGetUniformOffsetEXT = platform.createExtensionFunction( 
	'glGetUniformOffsetEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLintptr, 
	argTypes=(constants.GLuint, constants.GLint,),
	doc = 'glGetUniformOffsetEXT( GLuint(program), GLint(location) ) -> constants.GLintptr',
	argNames = ('program', 'location',),
)


def glInitBindableUniformEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.point_parameters

Overview (from the spec)
	
	    This extension supports additional geometric characteristics of points. It
	    can be used to render particles or tiny light sources, commonly referred
	    as "Light points".
	
	    The raster brightness of a point is a function of the point area, point
	    color, point transparency, and the response of the display's electron gun
	    and phosphor. The point area and the point transparency are derived from the
	    point size, currently provided with the <size> parameter of glPointSize.
	
	    The primary motivation is to allow the size of a point to be affected by
	    distance attenuation. When distance attenuation has an effect, the final
	    point size decreases as the distance of the point from the eye increases.
	
	    The secondary motivation is a mean to control the mapping from the point
	    size to the raster point area and point transparency. This is done in order
	    to increase the dynamic range of the raster brightness of points. In other
	    words, the alpha component of a point may be decreased (and its transparency
	    increased) as its area shrinks below a defined threshold.
	
	    This extension defines a derived point size to be closely related to point
	    brightness. The brightness of a point is given by:
	
				1
		dist_atten(d) = -------------------
				a + b * d + c * d^2
	
		brightness(Pe) = Brightness * dist_atten(|Pe|)
	
	    where 'Pe' is the point in eye coordinates, and 'Brightness' is some initial
	    value proportional to the square of the size provided with glPointSize. Here
	    we simplify the raster brightness to be a function of the rasterized point
	    area and point transparency.
	
			    brightness(Pe)			brightness(Pe) >= Threshold_Area
		area(Pe) =
			    Threshold_Area			Otherwise
	
		factor(Pe) = brightness(Pe)/Threshold_Area
	
		alpha(Pe) = Alpha * factor(Pe)
	
	    where 'Alpha' comes with the point color (possibly modified by lighting).
	
	    'Threshold_Area' above is in area units. Thus, it is proportional to the
	    square of the threshold provided by the programmer through this extension.
	
	    The new point size derivation method applies to all points, while the
	    threshold applies to multisample points only.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/point_parameters.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_point_parameters'
GL_POINT_SIZE_MIN_EXT = constant.Constant( 'GL_POINT_SIZE_MIN_EXT', 0x8126 )
GL_POINT_SIZE_MAX_EXT = constant.Constant( 'GL_POINT_SIZE_MAX_EXT', 0x8127 )
GL_POINT_FADE_THRESHOLD_SIZE_EXT = constant.Constant( 'GL_POINT_FADE_THRESHOLD_SIZE_EXT', 0x8128 )
GL_DISTANCE_ATTENUATION_EXT = constant.Constant( 'GL_DISTANCE_ATTENUATION_EXT', 0x8129 )
glPointParameterfEXT = platform.createExtensionFunction( 
	'glPointParameterfEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLfloat,),
	doc = 'glPointParameterfEXT( GLenum(pname), GLfloat(param) ) -> None',
	argNames = ('pname', 'param',),
)

glPointParameterfvEXT = platform.createExtensionFunction( 
	'glPointParameterfvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLfloatArray,),
	doc = 'glPointParameterfvEXT( GLenum(pname), GLfloatArray(params) ) -> None',
	argNames = ('pname', 'params',),
)


def glInitPointParametersEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""Constants for OpenGL.GLX

Automatically generated by the generateraw script, do not edit!
"""

from ctypes import *
from OpenGL import platform, arrays
from OpenGL.constant import Constant
from OpenGL import constants as GLconstants
GLvoid = GLconstants.GLvoid

STRING = c_char_p
from OpenGL.constants import GL_BYTE
from OpenGL.constants import GL_DOUBLE
from OpenGL.constants import GL_FALSE
from OpenGL.constants import GL_FLOAT
from OpenGL.constants import GL_INT
from OpenGL.constants import GL_SHORT
from OpenGL.constants import GL_TRUE
from OpenGL.constants import GL_UNSIGNED_BYTE
from OpenGL.constants import GL_UNSIGNED_INT
from OpenGL.constants import GL_UNSIGNED_SHORT


CWX = Constant( 'CWX', 1)
CWY = Constant( 'CWY', 2)
FUNCPROTO = Constant( 'FUNCPROTO', 15)
GLX_3DFX_FULLSCREEN_MODE_MESA = Constant( 'GLX_3DFX_FULLSCREEN_MODE_MESA', 2)
GLX_3DFX_WINDOW_MODE_MESA = Constant( 'GLX_3DFX_WINDOW_MODE_MESA', 1)
GLX_ACCUM_ALPHA_SIZE = Constant( 'GLX_ACCUM_ALPHA_SIZE', 17)
GLX_ACCUM_BLUE_SIZE = Constant( 'GLX_ACCUM_BLUE_SIZE', 16)
GLX_ACCUM_BUFFER_BIT = Constant( 'GLX_ACCUM_BUFFER_BIT', 128)
GLX_ACCUM_BUFFER_BIT_SGIX = Constant( 'GLX_ACCUM_BUFFER_BIT_SGIX', 128)
GLX_ACCUM_GREEN_SIZE = Constant( 'GLX_ACCUM_GREEN_SIZE', 15)
GLX_ACCUM_RED_SIZE = Constant( 'GLX_ACCUM_RED_SIZE', 14)
GLX_ALPHA_SIZE = Constant( 'GLX_ALPHA_SIZE', 11)
GLX_AUX0_EXT = Constant( 'GLX_AUX0_EXT', 8418)
GLX_AUX1_EXT = Constant( 'GLX_AUX1_EXT', 8419)
GLX_AUX2_EXT = Constant( 'GLX_AUX2_EXT', 8420)
GLX_AUX3_EXT = Constant( 'GLX_AUX3_EXT', 8421)
GLX_AUX4_EXT = Constant( 'GLX_AUX4_EXT', 8422)
GLX_AUX5_EXT = Constant( 'GLX_AUX5_EXT', 8423)
GLX_AUX6_EXT = Constant( 'GLX_AUX6_EXT', 8424)
GLX_AUX7_EXT = Constant( 'GLX_AUX7_EXT', 8425)
GLX_AUX8_EXT = Constant( 'GLX_AUX8_EXT', 8426)
GLX_AUX9_EXT = Constant( 'GLX_AUX9_EXT', 8427)
GLX_AUX_BUFFERS = Constant( 'GLX_AUX_BUFFERS', 7)
GLX_AUX_BUFFERS_BIT = Constant( 'GLX_AUX_BUFFERS_BIT', 16)
GLX_AUX_BUFFERS_BIT_SGIX = Constant( 'GLX_AUX_BUFFERS_BIT_SGIX', 16)
GLX_BACK_LEFT_BUFFER_BIT = Constant( 'GLX_BACK_LEFT_BUFFER_BIT', 4)
GLX_BACK_LEFT_BUFFER_BIT_SGIX = Constant( 'GLX_BACK_LEFT_BUFFER_BIT_SGIX', 4)
GLX_BACK_LEFT_EXT = Constant( 'GLX_BACK_LEFT_EXT', 8416)
GLX_BACK_RIGHT_BUFFER_BIT = Constant( 'GLX_BACK_RIGHT_BUFFER_BIT', 8)
GLX_BACK_RIGHT_BUFFER_BIT_SGIX = Constant( 'GLX_BACK_RIGHT_BUFFER_BIT_SGIX', 8)
GLX_BACK_RIGHT_EXT = Constant( 'GLX_BACK_RIGHT_EXT', 8417)
GLX_BAD_ATTRIBUTE = Constant( 'GLX_BAD_ATTRIBUTE', 2)
GLX_BAD_CONTEXT = Constant( 'GLX_BAD_CONTEXT', 5)
GLX_BAD_ENUM = Constant( 'GLX_BAD_ENUM', 7)
GLX_BAD_HYPERPIPE_CONFIG_SGIX = Constant( 'GLX_BAD_HYPERPIPE_CONFIG_SGIX', 91)
GLX_BAD_HYPERPIPE_SGIX = Constant( 'GLX_BAD_HYPERPIPE_SGIX', 92)
GLX_BAD_SCREEN = Constant( 'GLX_BAD_SCREEN', 1)
GLX_BAD_VALUE = Constant( 'GLX_BAD_VALUE', 6)
GLX_BAD_VISUAL = Constant( 'GLX_BAD_VISUAL', 4)
GLX_BIND_TO_MIPMAP_TEXTURE_EXT = Constant( 'GLX_BIND_TO_MIPMAP_TEXTURE_EXT', 8402)
GLX_BIND_TO_TEXTURE_RGBA_EXT = Constant( 'GLX_BIND_TO_TEXTURE_RGBA_EXT', 8401)
GLX_BIND_TO_TEXTURE_RGB_EXT = Constant( 'GLX_BIND_TO_TEXTURE_RGB_EXT', 8400)
GLX_BIND_TO_TEXTURE_TARGETS_EXT = Constant( 'GLX_BIND_TO_TEXTURE_TARGETS_EXT', 8403)
GLX_BLENDED_RGBA_SGIS = Constant( 'GLX_BLENDED_RGBA_SGIS', 32805)
GLX_BLUE_SIZE = Constant( 'GLX_BLUE_SIZE', 10)
GLX_BUFFER_CLOBBER_MASK_SGIX = Constant( 'GLX_BUFFER_CLOBBER_MASK_SGIX', 134217728)
GLX_BUFFER_SIZE = Constant( 'GLX_BUFFER_SIZE', 2)
GLX_COLOR_INDEX_BIT = Constant( 'GLX_COLOR_INDEX_BIT', 2)
GLX_COLOR_INDEX_BIT_SGIX = Constant( 'GLX_COLOR_INDEX_BIT_SGIX', 2)
GLX_COLOR_INDEX_TYPE = Constant( 'GLX_COLOR_INDEX_TYPE', 32789)
GLX_COLOR_INDEX_TYPE_SGIX = Constant( 'GLX_COLOR_INDEX_TYPE_SGIX', 32789)
GLX_CONFIG_CAVEAT = Constant( 'GLX_CONFIG_CAVEAT', 32)
GLX_DAMAGED = Constant( 'GLX_DAMAGED', 32800)
GLX_DAMAGED_SGIX = Constant( 'GLX_DAMAGED_SGIX', 32800)
GLX_DEPTH_BUFFER_BIT = Constant( 'GLX_DEPTH_BUFFER_BIT', 32)
GLX_DEPTH_BUFFER_BIT_SGIX = Constant( 'GLX_DEPTH_BUFFER_BIT_SGIX', 32)
GLX_DEPTH_SIZE = Constant( 'GLX_DEPTH_SIZE', 12)
GLX_DIGITAL_MEDIA_PBUFFER_SGIX = Constant( 'GLX_DIGITAL_MEDIA_PBUFFER_SGIX', 32804)
GLX_DIRECT_COLOR = Constant( 'GLX_DIRECT_COLOR', 32771)
GLX_DIRECT_COLOR_EXT = Constant( 'GLX_DIRECT_COLOR_EXT', 32771)
GLX_DONT_CARE = Constant( 'GLX_DONT_CARE', 4294967295)
GLX_DOUBLEBUFFER = Constant( 'GLX_DOUBLEBUFFER', 5)
GLX_DRAWABLE_TYPE = Constant( 'GLX_DRAWABLE_TYPE', 32784)
GLX_DRAWABLE_TYPE_SGIX = Constant( 'GLX_DRAWABLE_TYPE_SGIX', 32784)
GLX_EVENT_MASK = Constant( 'GLX_EVENT_MASK', 32799)
GLX_EVENT_MASK_SGIX = Constant( 'GLX_EVENT_MASK_SGIX', 32799)
GLX_EXTENSIONS = Constant( 'GLX_EXTENSIONS', 3)
GLX_EXTENSION_NAME = Constant( 'GLX_EXTENSION_NAME', 'GLX')
GLX_FBCONFIG_ID = Constant( 'GLX_FBCONFIG_ID', 32787)
GLX_FBCONFIG_ID_SGIX = Constant( 'GLX_FBCONFIG_ID_SGIX', 32787)
GLX_FLOAT_COMPONENTS_NV = Constant( 'GLX_FLOAT_COMPONENTS_NV', 8368)
GLX_FRONT_LEFT_BUFFER_BIT = Constant( 'GLX_FRONT_LEFT_BUFFER_BIT', 1)
GLX_FRONT_LEFT_BUFFER_BIT_SGIX = Constant( 'GLX_FRONT_LEFT_BUFFER_BIT_SGIX', 1)
GLX_FRONT_LEFT_EXT = Constant( 'GLX_FRONT_LEFT_EXT', 8414)
GLX_FRONT_RIGHT_BUFFER_BIT = Constant( 'GLX_FRONT_RIGHT_BUFFER_BIT', 2)
GLX_FRONT_RIGHT_BUFFER_BIT_SGIX = Constant( 'GLX_FRONT_RIGHT_BUFFER_BIT_SGIX', 2)
GLX_FRONT_RIGHT_EXT = Constant( 'GLX_FRONT_RIGHT_EXT', 8415)
GLX_GLXEXT_VERSION = Constant( 'GLX_GLXEXT_VERSION', 11)
GLX_GRAY_SCALE = Constant( 'GLX_GRAY_SCALE', 32774)
GLX_GRAY_SCALE_EXT = Constant( 'GLX_GRAY_SCALE_EXT', 32774)
GLX_GREEN_SIZE = Constant( 'GLX_GREEN_SIZE', 9)
GLX_HEIGHT = Constant( 'GLX_HEIGHT', 32798)
GLX_HEIGHT_SGIX = Constant( 'GLX_HEIGHT_SGIX', 32798)
GLX_HYPERPIPE_DISPLAY_PIPE_SGIX = Constant( 'GLX_HYPERPIPE_DISPLAY_PIPE_SGIX', 1)
GLX_HYPERPIPE_ID_SGIX = Constant( 'GLX_HYPERPIPE_ID_SGIX', 32816)
GLX_HYPERPIPE_PIPE_NAME_LENGTH_SGIX = Constant( 'GLX_HYPERPIPE_PIPE_NAME_LENGTH_SGIX', 80)
GLX_HYPERPIPE_PIXEL_AVERAGE_SGIX = Constant( 'GLX_HYPERPIPE_PIXEL_AVERAGE_SGIX', 4)
GLX_HYPERPIPE_RENDER_PIPE_SGIX = Constant( 'GLX_HYPERPIPE_RENDER_PIPE_SGIX', 2)
GLX_HYPERPIPE_STEREO_SGIX = Constant( 'GLX_HYPERPIPE_STEREO_SGIX', 3)
GLX_LARGEST_PBUFFER = Constant( 'GLX_LARGEST_PBUFFER', 32796)
GLX_LARGEST_PBUFFER_SGIX = Constant( 'GLX_LARGEST_PBUFFER_SGIX', 32796)
GLX_LEVEL = Constant( 'GLX_LEVEL', 3)
GLX_MAX_PBUFFER_HEIGHT = Constant( 'GLX_MAX_PBUFFER_HEIGHT', 32791)
GLX_MAX_PBUFFER_HEIGHT_SGIX = Constant( 'GLX_MAX_PBUFFER_HEIGHT_SGIX', 32791)
GLX_MAX_PBUFFER_PIXELS = Constant( 'GLX_MAX_PBUFFER_PIXELS', 32792)
GLX_MAX_PBUFFER_PIXELS_SGIX = Constant( 'GLX_MAX_PBUFFER_PIXELS_SGIX', 32792)
GLX_MAX_PBUFFER_WIDTH = Constant( 'GLX_MAX_PBUFFER_WIDTH', 32790)
GLX_MAX_PBUFFER_WIDTH_SGIX = Constant( 'GLX_MAX_PBUFFER_WIDTH_SGIX', 32790)
GLX_MIPMAP_TEXTURE_EXT = Constant( 'GLX_MIPMAP_TEXTURE_EXT', 8407)
GLX_MULTISAMPLE_SUB_RECT_HEIGHT_SGIS = Constant( 'GLX_MULTISAMPLE_SUB_RECT_HEIGHT_SGIS', 32807)
GLX_MULTISAMPLE_SUB_RECT_WIDTH_SGIS = Constant( 'GLX_MULTISAMPLE_SUB_RECT_WIDTH_SGIS', 32806)
GLX_NONE = Constant( 'GLX_NONE', 32768)
GLX_NONE_EXT = Constant( 'GLX_NONE_EXT', 32768)
GLX_NON_CONFORMANT_CONFIG = Constant( 'GLX_NON_CONFORMANT_CONFIG', 32781)
GLX_NON_CONFORMANT_VISUAL_EXT = Constant( 'GLX_NON_CONFORMANT_VISUAL_EXT', 32781)
GLX_NO_EXTENSION = Constant( 'GLX_NO_EXTENSION', 3)
GLX_OPTIMAL_PBUFFER_HEIGHT_SGIX = Constant( 'GLX_OPTIMAL_PBUFFER_HEIGHT_SGIX', 32794)
GLX_OPTIMAL_PBUFFER_WIDTH_SGIX = Constant( 'GLX_OPTIMAL_PBUFFER_WIDTH_SGIX', 32793)
GLX_PBUFFER = Constant( 'GLX_PBUFFER', 32803)
GLX_PBUFFER_BIT = Constant( 'GLX_PBUFFER_BIT', 4)
GLX_PBUFFER_BIT_SGIX = Constant( 'GLX_PBUFFER_BIT_SGIX', 4)
GLX_PBUFFER_CLOBBER_MASK = Constant( 'GLX_PBUFFER_CLOBBER_MASK', 134217728)
GLX_PBUFFER_HEIGHT = Constant( 'GLX_PBUFFER_HEIGHT', 32832)
GLX_PBUFFER_SGIX = Constant( 'GLX_PBUFFER_SGIX', 32803)
GLX_PBUFFER_WIDTH = Constant( 'GLX_PBUFFER_WIDTH', 32833)
GLX_PIPE_RECT_LIMITS_SGIX = Constant( 'GLX_PIPE_RECT_LIMITS_SGIX', 2)
GLX_PIPE_RECT_SGIX = Constant( 'GLX_PIPE_RECT_SGIX', 1)
GLX_PIXMAP_BIT = Constant( 'GLX_PIXMAP_BIT', 2)
GLX_PIXMAP_BIT_SGIX = Constant( 'GLX_PIXMAP_BIT_SGIX', 2)
GLX_PRESERVED_CONTENTS = Constant( 'GLX_PRESERVED_CONTENTS', 32795)
GLX_PRESERVED_CONTENTS_SGIX = Constant( 'GLX_PRESERVED_CONTENTS_SGIX', 32795)
GLX_PSEUDO_COLOR = Constant( 'GLX_PSEUDO_COLOR', 32772)
GLX_PSEUDO_COLOR_EXT = Constant( 'GLX_PSEUDO_COLOR_EXT', 32772)
GLX_RED_SIZE = Constant( 'GLX_RED_SIZE', 8)
GLX_RENDER_TYPE = Constant( 'GLX_RENDER_TYPE', 32785)
GLX_RENDER_TYPE_SGIX = Constant( 'GLX_RENDER_TYPE_SGIX', 32785)
GLX_RGBA = Constant( 'GLX_RGBA', 4)
GLX_RGBA_BIT = Constant( 'GLX_RGBA_BIT', 1)
GLX_RGBA_BIT_SGIX = Constant( 'GLX_RGBA_BIT_SGIX', 1)
GLX_RGBA_FLOAT_BIT_ARB = Constant( 'GLX_RGBA_FLOAT_BIT_ARB', 4)
GLX_RGBA_FLOAT_TYPE_ARB = Constant( 'GLX_RGBA_FLOAT_TYPE_ARB', 8377)
GLX_RGBA_TYPE = Constant( 'GLX_RGBA_TYPE', 32788)
GLX_RGBA_TYPE_SGIX = Constant( 'GLX_RGBA_TYPE_SGIX', 32788)
GLX_SAMPLES = Constant( 'GLX_SAMPLES', 100001)
GLX_SAMPLES_3DFX = Constant( 'GLX_SAMPLES_3DFX', 32849)
GLX_SAMPLES_ARB = Constant( 'GLX_SAMPLES_ARB', 100001)
GLX_SAMPLES_SGIS = Constant( 'GLX_SAMPLES_SGIS', 100001)
GLX_SAMPLE_BUFFERS = Constant( 'GLX_SAMPLE_BUFFERS', 100000)
GLX_SAMPLE_BUFFERS_3DFX = Constant( 'GLX_SAMPLE_BUFFERS_3DFX', 32848)
GLX_SAMPLE_BUFFERS_ARB = Constant( 'GLX_SAMPLE_BUFFERS_ARB', 100000)
GLX_SAMPLE_BUFFERS_BIT_SGIX = Constant( 'GLX_SAMPLE_BUFFERS_BIT_SGIX', 256)
GLX_SAMPLE_BUFFERS_SGIS = Constant( 'GLX_SAMPLE_BUFFERS_SGIS', 100000)
GLX_SAVED = Constant( 'GLX_SAVED', 32801)
GLX_SAVED_SGIX = Constant( 'GLX_SAVED_SGIX', 32801)
GLX_SCREEN = Constant( 'GLX_SCREEN', 32780)
GLX_SCREEN_EXT = Constant( 'GLX_SCREEN_EXT', 32780)
GLX_SHARE_CONTEXT_EXT = Constant( 'GLX_SHARE_CONTEXT_EXT', 32778)
GLX_SLOW_CONFIG = Constant( 'GLX_SLOW_CONFIG', 32769)
GLX_SLOW_VISUAL_EXT = Constant( 'GLX_SLOW_VISUAL_EXT', 32769)
GLX_STATIC_COLOR = Constant( 'GLX_STATIC_COLOR', 32773)
GLX_STATIC_COLOR_EXT = Constant( 'GLX_STATIC_COLOR_EXT', 32773)
GLX_STATIC_GRAY = Constant( 'GLX_STATIC_GRAY', 32775)
GLX_STATIC_GRAY_EXT = Constant( 'GLX_STATIC_GRAY_EXT', 32775)
GLX_STENCIL_BUFFER_BIT = Constant( 'GLX_STENCIL_BUFFER_BIT', 64)
GLX_STENCIL_BUFFER_BIT_SGIX = Constant( 'GLX_STENCIL_BUFFER_BIT_SGIX', 64)
GLX_STENCIL_SIZE = Constant( 'GLX_STENCIL_SIZE', 13)
GLX_STEREO = Constant( 'GLX_STEREO', 6)
GLX_SWAP_COPY_OML = Constant( 'GLX_SWAP_COPY_OML', 32866)
GLX_SWAP_EXCHANGE_OML = Constant( 'GLX_SWAP_EXCHANGE_OML', 32865)
GLX_SWAP_METHOD_OML = Constant( 'GLX_SWAP_METHOD_OML', 32864)
GLX_SWAP_UNDEFINED_OML = Constant( 'GLX_SWAP_UNDEFINED_OML', 32867)
GLX_SYNC_FRAME_SGIX = Constant( 'GLX_SYNC_FRAME_SGIX', 0)
GLX_SYNC_SWAP_SGIX = Constant( 'GLX_SYNC_SWAP_SGIX', 1)
GLX_TEXTURE_1D_BIT_EXT = Constant( 'GLX_TEXTURE_1D_BIT_EXT', 1)
GLX_TEXTURE_1D_EXT = Constant( 'GLX_TEXTURE_1D_EXT', 8411)
GLX_TEXTURE_2D_BIT_EXT = Constant( 'GLX_TEXTURE_2D_BIT_EXT', 2)
GLX_TEXTURE_2D_EXT = Constant( 'GLX_TEXTURE_2D_EXT', 8412)
GLX_TEXTURE_FORMAT_EXT = Constant( 'GLX_TEXTURE_FORMAT_EXT', 8405)
GLX_TEXTURE_FORMAT_NONE_EXT = Constant( 'GLX_TEXTURE_FORMAT_NONE_EXT', 8408)
GLX_TEXTURE_FORMAT_RGBA_EXT = Constant( 'GLX_TEXTURE_FORMAT_RGBA_EXT', 8410)
GLX_TEXTURE_FORMAT_RGB_EXT = Constant( 'GLX_TEXTURE_FORMAT_RGB_EXT', 8409)
GLX_TEXTURE_RECTANGLE_BIT_EXT = Constant( 'GLX_TEXTURE_RECTANGLE_BIT_EXT', 4)
GLX_TEXTURE_RECTANGLE_EXT = Constant( 'GLX_TEXTURE_RECTANGLE_EXT', 8413)
GLX_TEXTURE_TARGET_EXT = Constant( 'GLX_TEXTURE_TARGET_EXT', 8406)
GLX_TRANSPARENT_ALPHA_VALUE = Constant( 'GLX_TRANSPARENT_ALPHA_VALUE', 40)
GLX_TRANSPARENT_ALPHA_VALUE_EXT = Constant( 'GLX_TRANSPARENT_ALPHA_VALUE_EXT', 40)
GLX_TRANSPARENT_BLUE_VALUE = Constant( 'GLX_TRANSPARENT_BLUE_VALUE', 39)
GLX_TRANSPARENT_BLUE_VALUE_EXT = Constant( 'GLX_TRANSPARENT_BLUE_VALUE_EXT', 39)
GLX_TRANSPARENT_GREEN_VALUE = Constant( 'GLX_TRANSPARENT_GREEN_VALUE', 38)
GLX_TRANSPARENT_GREEN_VALUE_EXT = Constant( 'GLX_TRANSPARENT_GREEN_VALUE_EXT', 38)
GLX_TRANSPARENT_INDEX = Constant( 'GLX_TRANSPARENT_INDEX', 32777)
GLX_TRANSPARENT_INDEX_EXT = Constant( 'GLX_TRANSPARENT_INDEX_EXT', 32777)
GLX_TRANSPARENT_INDEX_VALUE = Constant( 'GLX_TRANSPARENT_INDEX_VALUE', 36)
GLX_TRANSPARENT_INDEX_VALUE_EXT = Constant( 'GLX_TRANSPARENT_INDEX_VALUE_EXT', 36)
GLX_TRANSPARENT_RED_VALUE = Constant( 'GLX_TRANSPARENT_RED_VALUE', 37)
GLX_TRANSPARENT_RED_VALUE_EXT = Constant( 'GLX_TRANSPARENT_RED_VALUE_EXT', 37)
GLX_TRANSPARENT_RGB = Constant( 'GLX_TRANSPARENT_RGB', 32776)
GLX_TRANSPARENT_RGB_EXT = Constant( 'GLX_TRANSPARENT_RGB_EXT', 32776)
GLX_TRANSPARENT_TYPE = Constant( 'GLX_TRANSPARENT_TYPE', 35)
GLX_TRANSPARENT_TYPE_EXT = Constant( 'GLX_TRANSPARENT_TYPE_EXT', 35)
GLX_TRUE_COLOR = Constant( 'GLX_TRUE_COLOR', 32770)
GLX_TRUE_COLOR_EXT = Constant( 'GLX_TRUE_COLOR_EXT', 32770)
GLX_USE_GL = Constant( 'GLX_USE_GL', 1)
GLX_VENDOR = Constant( 'GLX_VENDOR', 1)
GLX_VERSION = Constant( 'GLX_VERSION', 2)
GLX_VERSION_1_1 = Constant( 'GLX_VERSION_1_1', 1)
GLX_VERSION_1_2 = Constant( 'GLX_VERSION_1_2', 1)
GLX_VERSION_1_3 = Constant( 'GLX_VERSION_1_3', 1)
GLX_VERSION_1_4 = Constant( 'GLX_VERSION_1_4', 1)
GLX_VISUAL_CAVEAT_EXT = Constant( 'GLX_VISUAL_CAVEAT_EXT', 32)
GLX_VISUAL_ID = Constant( 'GLX_VISUAL_ID', 32779)
GLX_VISUAL_ID_EXT = Constant( 'GLX_VISUAL_ID_EXT', 32779)
GLX_VISUAL_SELECT_GROUP_SGIX = Constant( 'GLX_VISUAL_SELECT_GROUP_SGIX', 32808)
GLX_WIDTH = Constant( 'GLX_WIDTH', 32797)
GLX_WIDTH_SGIX = Constant( 'GLX_WIDTH_SGIX', 32797)
GLX_WINDOW = Constant( 'GLX_WINDOW', 32802)
GLX_WINDOW_BIT = Constant( 'GLX_WINDOW_BIT', 1)
GLX_WINDOW_BIT_SGIX = Constant( 'GLX_WINDOW_BIT_SGIX', 1)
GLX_WINDOW_SGIX = Constant( 'GLX_WINDOW_SGIX', 32802)
GLX_X_RENDERABLE = Constant( 'GLX_X_RENDERABLE', 32786)
GLX_X_RENDERABLE_SGIX = Constant( 'GLX_X_RENDERABLE_SGIX', 32786)
GLX_X_VISUAL_TYPE = Constant( 'GLX_X_VISUAL_TYPE', 34)
GLX_X_VISUAL_TYPE_EXT = Constant( 'GLX_X_VISUAL_TYPE_EXT', 34)
GLX_Y_INVERTED_EXT = Constant( 'GLX_Y_INVERTED_EXT', 8404)
GL_ALL_CLIENT_ATTRIB_BITS = Constant( 'GL_ALL_CLIENT_ATTRIB_BITS', 4294967295)
GL_ALPHA_BLEND_EQUATION_ATI = Constant( 'GL_ALPHA_BLEND_EQUATION_ATI', 34877)
GL_DEBUG_ASSERT_MESA = Constant( 'GL_DEBUG_ASSERT_MESA', 34651)
GL_DEBUG_OBJECT_MESA = Constant( 'GL_DEBUG_OBJECT_MESA', 34649)
GL_DEBUG_PRINT_MESA = Constant( 'GL_DEBUG_PRINT_MESA', 34650)
GL_DEPTH_STENCIL_MESA = Constant( 'GL_DEPTH_STENCIL_MESA', 34640)
GL_FRAGMENT_PROGRAM_CALLBACK_DATA_MESA = Constant( 'GL_FRAGMENT_PROGRAM_CALLBACK_DATA_MESA', 35763)
GL_FRAGMENT_PROGRAM_CALLBACK_FUNC_MESA = Constant( 'GL_FRAGMENT_PROGRAM_CALLBACK_FUNC_MESA', 35762)
GL_FRAGMENT_PROGRAM_CALLBACK_MESA = Constant( 'GL_FRAGMENT_PROGRAM_CALLBACK_MESA', 35761)
GL_FRAGMENT_PROGRAM_POSITION_MESA = Constant( 'GL_FRAGMENT_PROGRAM_POSITION_MESA', 35760)
GL_TRACE_ALL_BITS_MESA = Constant( 'GL_TRACE_ALL_BITS_MESA', 65535)
GL_TRACE_ARRAYS_BIT_MESA = Constant( 'GL_TRACE_ARRAYS_BIT_MESA', 4)
GL_TRACE_ERRORS_BIT_MESA = Constant( 'GL_TRACE_ERRORS_BIT_MESA', 32)
GL_TRACE_MASK_MESA = Constant( 'GL_TRACE_MASK_MESA', 34645)
GL_TRACE_NAME_MESA = Constant( 'GL_TRACE_NAME_MESA', 34646)
GL_TRACE_OPERATIONS_BIT_MESA = Constant( 'GL_TRACE_OPERATIONS_BIT_MESA', 1)
GL_TRACE_PIXELS_BIT_MESA = Constant( 'GL_TRACE_PIXELS_BIT_MESA', 16)
GL_TRACE_PRIMITIVES_BIT_MESA = Constant( 'GL_TRACE_PRIMITIVES_BIT_MESA', 2)
GL_TRACE_TEXTURES_BIT_MESA = Constant( 'GL_TRACE_TEXTURES_BIT_MESA', 8)
GL_UNSIGNED_INT_24_8_MESA = Constant( 'GL_UNSIGNED_INT_24_8_MESA', 34641)
GL_UNSIGNED_INT_8_24_REV_MESA = Constant( 'GL_UNSIGNED_INT_8_24_REV_MESA', 34642)
GL_UNSIGNED_SHORT_15_1_MESA = Constant( 'GL_UNSIGNED_SHORT_15_1_MESA', 34643)
GL_UNSIGNED_SHORT_1_15_REV_MESA = Constant( 'GL_UNSIGNED_SHORT_1_15_REV_MESA', 34644)
GL_VERTEX_PROGRAM_CALLBACK_DATA_MESA = Constant( 'GL_VERTEX_PROGRAM_CALLBACK_DATA_MESA', 35767)
GL_VERTEX_PROGRAM_CALLBACK_FUNC_MESA = Constant( 'GL_VERTEX_PROGRAM_CALLBACK_FUNC_MESA', 35766)
GL_VERTEX_PROGRAM_CALLBACK_MESA = Constant( 'GL_VERTEX_PROGRAM_CALLBACK_MESA', 35765)
GL_VERTEX_PROGRAM_POSITION_MESA = Constant( 'GL_VERTEX_PROGRAM_POSITION_MESA', 35764)
XCNOENT = Constant( 'XCNOENT', 2)
XCNOMEM = Constant( 'XCNOMEM', 1)
XCSUCCESS = Constant( 'XCSUCCESS', 0)
XK_0 = Constant( 'XK_0', 48)
XK_1 = Constant( 'XK_1', 49)
XK_2 = Constant( 'XK_2', 50)
XK_3 = Constant( 'XK_3', 51)
XK_4 = Constant( 'XK_4', 52)
XK_5 = Constant( 'XK_5', 53)
XK_6 = Constant( 'XK_6', 54)
XK_7 = Constant( 'XK_7', 55)
XK_8 = Constant( 'XK_8', 56)
XK_9 = Constant( 'XK_9', 57)
XK_A = Constant( 'XK_A', 65)
XK_AE = Constant( 'XK_AE', 198)
XK_B = Constant( 'XK_B', 66)
XK_C = Constant( 'XK_C', 67)
XK_D = Constant( 'XK_D', 68)
XK_E = Constant( 'XK_E', 69)
XK_ENG = Constant( 'XK_ENG', 957)
XK_ETH = Constant( 'XK_ETH', 208)
XK_F = Constant( 'XK_F', 70)
XK_F1 = Constant( 'XK_F1', 65470)
XK_F10 = Constant( 'XK_F10', 65479)
XK_F11 = Constant( 'XK_F11', 65480)
XK_F12 = Constant( 'XK_F12', 65481)
XK_F13 = Constant( 'XK_F13', 65482)
XK_F14 = Constant( 'XK_F14', 65483)
XK_F15 = Constant( 'XK_F15', 65484)
XK_F16 = Constant( 'XK_F16', 65485)
XK_F17 = Constant( 'XK_F17', 65486)
XK_F18 = Constant( 'XK_F18', 65487)
XK_F19 = Constant( 'XK_F19', 65488)
XK_F2 = Constant( 'XK_F2', 65471)
XK_F20 = Constant( 'XK_F20', 65489)
XK_F21 = Constant( 'XK_F21', 65490)
XK_F22 = Constant( 'XK_F22', 65491)
XK_F23 = Constant( 'XK_F23', 65492)
XK_F24 = Constant( 'XK_F24', 65493)
XK_F25 = Constant( 'XK_F25', 65494)
XK_F26 = Constant( 'XK_F26', 65495)
XK_F27 = Constant( 'XK_F27', 65496)
XK_F28 = Constant( 'XK_F28', 65497)
XK_F29 = Constant( 'XK_F29', 65498)
XK_F3 = Constant( 'XK_F3', 65472)
XK_F30 = Constant( 'XK_F30', 65499)
XK_F31 = Constant( 'XK_F31', 65500)
XK_F32 = Constant( 'XK_F32', 65501)
XK_F33 = Constant( 'XK_F33', 65502)
XK_F34 = Constant( 'XK_F34', 65503)
XK_F35 = Constant( 'XK_F35', 65504)
XK_F4 = Constant( 'XK_F4', 65473)
XK_F5 = Constant( 'XK_F5', 65474)
XK_F6 = Constant( 'XK_F6', 65475)
XK_F7 = Constant( 'XK_F7', 65476)
XK_F8 = Constant( 'XK_F8', 65477)
XK_F9 = Constant( 'XK_F9', 65478)
XK_G = Constant( 'XK_G', 71)
XK_H = Constant( 'XK_H', 72)
XK_I = Constant( 'XK_I', 73)
XK_J = Constant( 'XK_J', 74)
XK_K = Constant( 'XK_K', 75)
XK_KP_0 = Constant( 'XK_KP_0', 65456)
XK_KP_1 = Constant( 'XK_KP_1', 65457)
XK_KP_2 = Constant( 'XK_KP_2', 65458)
XK_KP_3 = Constant( 'XK_KP_3', 65459)
XK_KP_4 = Constant( 'XK_KP_4', 65460)
XK_KP_5 = Constant( 'XK_KP_5', 65461)
XK_KP_6 = Constant( 'XK_KP_6', 65462)
XK_KP_7 = Constant( 'XK_KP_7', 65463)
XK_KP_8 = Constant( 'XK_KP_8', 65464)
XK_KP_9 = Constant( 'XK_KP_9', 65465)
XK_KP_F1 = Constant( 'XK_KP_F1', 65425)
XK_KP_F2 = Constant( 'XK_KP_F2', 65426)
XK_KP_F3 = Constant( 'XK_KP_F3', 65427)
XK_KP_F4 = Constant( 'XK_KP_F4', 65428)
XK_L = Constant( 'XK_L', 76)
XK_L1 = Constant( 'XK_L1', 65480)
XK_L10 = Constant( 'XK_L10', 65489)
XK_L2 = Constant( 'XK_L2', 65481)
XK_L3 = Constant( 'XK_L3', 65482)
XK_L4 = Constant( 'XK_L4', 65483)
XK_L5 = Constant( 'XK_L5', 65484)
XK_L6 = Constant( 'XK_L6', 65485)
XK_L7 = Constant( 'XK_L7', 65486)
XK_L8 = Constant( 'XK_L8', 65487)
XK_L9 = Constant( 'XK_L9', 65488)
XK_M = Constant( 'XK_M', 77)
XK_N = Constant( 'XK_N', 78)
XK_O = Constant( 'XK_O', 79)
XK_OE = Constant( 'XK_OE', 5052)
XK_P = Constant( 'XK_P', 80)
XK_Q = Constant( 'XK_Q', 81)
XK_R = Constant( 'XK_R', 82)
XK_R1 = Constant( 'XK_R1', 65490)
XK_R10 = Constant( 'XK_R10', 65499)
XK_R11 = Constant( 'XK_R11', 65500)
XK_R12 = Constant( 'XK_R12', 65501)
XK_R13 = Constant( 'XK_R13', 65502)
XK_R14 = Constant( 'XK_R14', 65503)
XK_R15 = Constant( 'XK_R15', 65504)
XK_R2 = Constant( 'XK_R2', 65491)
XK_R3 = Constant( 'XK_R3', 65492)
XK_R4 = Constant( 'XK_R4', 65493)
XK_R5 = Constant( 'XK_R5', 65494)
XK_R6 = Constant( 'XK_R6', 65495)
XK_R7 = Constant( 'XK_R7', 65496)
XK_R8 = Constant( 'XK_R8', 65497)
XK_R9 = Constant( 'XK_R9', 65498)
XK_S = Constant( 'XK_S', 83)
XK_SCHWA = Constant( 'XK_SCHWA', 16777615)
XK_T = Constant( 'XK_T', 84)
XK_THORN = Constant( 'XK_THORN', 222)
XK_U = Constant( 'XK_U', 85)
XK_V = Constant( 'XK_V', 86)
XK_W = Constant( 'XK_W', 87)
XK_X = Constant( 'XK_X', 88)
XK_Y = Constant( 'XK_Y', 89)
XK_Z = Constant( 'XK_Z', 90)
X_HAVE_UTF8_STRING = Constant( 'X_HAVE_UTF8_STRING', 1)
X_PROTOCOL = Constant( 'X_PROTOCOL', 11)
X_PROTOCOL_REVISION = Constant( 'X_PROTOCOL_REVISION', 0)
_ATFILE_SOURCE = Constant( '_ATFILE_SOURCE', 1)
_BITS_PTHREADTYPES_H = Constant( '_BITS_PTHREADTYPES_H', 1)
_BITS_TYPESIZES_H = Constant( '_BITS_TYPESIZES_H', 1)
_BITS_TYPES_H = Constant( '_BITS_TYPES_H', 1)
_BITS_WCHAR_H = Constant( '_BITS_WCHAR_H', 1)
_BSD_SOURCE = Constant( '_BSD_SOURCE', 1)
_ENDIAN_H = Constant( '_ENDIAN_H', 1)
_FEATURES_H = Constant( '_FEATURES_H', 1)
_INTTYPES_H = Constant( '_INTTYPES_H', 1)
_ISOC99_SOURCE = Constant( '_ISOC99_SOURCE', 1)
_LARGEFILE64_SOURCE = Constant( '_LARGEFILE64_SOURCE', 1)
_LARGEFILE_SOURCE = Constant( '_LARGEFILE_SOURCE', 1)
_POSIX_C_SOURCE = Constant( '_POSIX_C_SOURCE', 199506)
_POSIX_SOURCE = Constant( '_POSIX_SOURCE', 1)
_SIGSET_NWORDS = Constant( '_SIGSET_NWORDS', 16)
_STDINT_H = Constant( '_STDINT_H', 1)
_STRUCT_TIMEVAL = Constant( '_STRUCT_TIMEVAL', 1)
_SVID_SOURCE = Constant( '_SVID_SOURCE', 1)
_SYS_CDEFS_H = Constant( '_SYS_CDEFS_H', 1)
_SYS_SELECT_H = Constant( '_SYS_SELECT_H', 1)
_SYS_SYSMACROS_H = Constant( '_SYS_SYSMACROS_H', 1)
_SYS_TYPES_H = Constant( '_SYS_TYPES_H', 1)
_XOPEN_SOURCE = Constant( '_XOPEN_SOURCE', 600)
_XOPEN_SOURCE_EXTENDED = Constant( '_XOPEN_SOURCE_EXTENDED', 1)
__BIG_ENDIAN = Constant( '__BIG_ENDIAN', 4321)
__BIT_TYPES_DEFINED__ = Constant( '__BIT_TYPES_DEFINED__', 1)
__FD_SETSIZE = Constant( '__FD_SETSIZE', 1024)
__GLIBC_HAVE_LONG_LONG = Constant( '__GLIBC_HAVE_LONG_LONG', 1)
__GLIBC_MINOR__ = Constant( '__GLIBC_MINOR__', 4)
__GLIBC__ = Constant( '__GLIBC__', 2)
__GNU_LIBRARY__ = Constant( '__GNU_LIBRARY__', 6)
__LITTLE_ENDIAN = Constant( '__LITTLE_ENDIAN', 1234)
__NFDBITS = Constant( '__NFDBITS', 64)
__PDP_ENDIAN = Constant( '__PDP_ENDIAN', 3412)
__PTHREAD_MUTEX_HAVE_PREV = Constant( '__PTHREAD_MUTEX_HAVE_PREV', 1)
__SIZEOF_PTHREAD_ATTR_T = Constant( '__SIZEOF_PTHREAD_ATTR_T', 56)
__SIZEOF_PTHREAD_BARRIERATTR_T = Constant( '__SIZEOF_PTHREAD_BARRIERATTR_T', 4)
__SIZEOF_PTHREAD_BARRIER_T = Constant( '__SIZEOF_PTHREAD_BARRIER_T', 32)
__SIZEOF_PTHREAD_CONDATTR_T = Constant( '__SIZEOF_PTHREAD_CONDATTR_T', 4)
__SIZEOF_PTHREAD_COND_T = Constant( '__SIZEOF_PTHREAD_COND_T', 48)
__SIZEOF_PTHREAD_MUTEXATTR_T = Constant( '__SIZEOF_PTHREAD_MUTEXATTR_T', 4)
__SIZEOF_PTHREAD_MUTEX_T = Constant( '__SIZEOF_PTHREAD_MUTEX_T', 40)
__SIZEOF_PTHREAD_RWLOCKATTR_T = Constant( '__SIZEOF_PTHREAD_RWLOCKATTR_T', 8)
__SIZEOF_PTHREAD_RWLOCK_T = Constant( '__SIZEOF_PTHREAD_RWLOCK_T', 56)
__STDC_IEC_559_COMPLEX__ = Constant( '__STDC_IEC_559_COMPLEX__', 1)
__STDC_IEC_559__ = Constant( '__STDC_IEC_559__', 1)
__STDC_ISO_10646__ = Constant( '__STDC_ISO_10646__', 200009)
__USE_ANSI = Constant( '__USE_ANSI', 1)
__USE_ATFILE = Constant( '__USE_ATFILE', 1)
__USE_BSD = Constant( '__USE_BSD', 1)
__USE_FORTIFY_LEVEL = Constant( '__USE_FORTIFY_LEVEL', 0)
__USE_GNU = Constant( '__USE_GNU', 1)
__USE_ISOC99 = Constant( '__USE_ISOC99', 1)
__USE_LARGEFILE = Constant( '__USE_LARGEFILE', 1)
__USE_LARGEFILE64 = Constant( '__USE_LARGEFILE64', 1)
__USE_MISC = Constant( '__USE_MISC', 1)
__USE_POSIX = Constant( '__USE_POSIX', 1)
__USE_POSIX199309 = Constant( '__USE_POSIX199309', 1)
__USE_POSIX199506 = Constant( '__USE_POSIX199506', 1)
__USE_POSIX2 = Constant( '__USE_POSIX2', 1)
__USE_SVID = Constant( '__USE_SVID', 1)
__USE_UNIX98 = Constant( '__USE_UNIX98', 1)
__USE_XOPEN = Constant( '__USE_XOPEN', 1)
__USE_XOPEN2K = Constant( '__USE_XOPEN2K', 1)
__USE_XOPEN_EXTENDED = Constant( '__USE_XOPEN_EXTENDED', 1)
__WCHAR_MAX = Constant( '__WCHAR_MAX', 2147483647)
__WCHAR_MIN = Constant( '__WCHAR_MIN', -2147483648)
__WORDSIZE = Constant( '__WORDSIZE', 64)
__WORDSIZE_COMPAT32 = Constant( '__WORDSIZE_COMPAT32', 1)
__all__ = [
	'CWX',
	'CWY',
	'FUNCPROTO',
	'GLX_3DFX_FULLSCREEN_MODE_MESA',
	'GLX_3DFX_WINDOW_MODE_MESA',
	'GLX_ACCUM_ALPHA_SIZE',
	'GLX_ACCUM_BLUE_SIZE',
	'GLX_ACCUM_BUFFER_BIT',
	'GLX_ACCUM_BUFFER_BIT_SGIX',
	'GLX_ACCUM_GREEN_SIZE',
	'GLX_ACCUM_RED_SIZE',
	'GLX_ALPHA_SIZE',
	'GLX_AUX0_EXT',
	'GLX_AUX1_EXT',
	'GLX_AUX2_EXT',
	'GLX_AUX3_EXT',
	'GLX_AUX4_EXT',
	'GLX_AUX5_EXT',
	'GLX_AUX6_EXT',
	'GLX_AUX7_EXT',
	'GLX_AUX8_EXT',
	'GLX_AUX9_EXT',
	'GLX_AUX_BUFFERS',
	'GLX_AUX_BUFFERS_BIT',
	'GLX_AUX_BUFFERS_BIT_SGIX',
	'GLX_BACK_LEFT_BUFFER_BIT',
	'GLX_BACK_LEFT_BUFFER_BIT_SGIX',
	'GLX_BACK_LEFT_EXT',
	'GLX_BACK_RIGHT_BUFFER_BIT',
	'GLX_BACK_RIGHT_BUFFER_BIT_SGIX',
	'GLX_BACK_RIGHT_EXT',
	'GLX_BAD_ATTRIBUTE',
	'GLX_BAD_CONTEXT',
	'GLX_BAD_ENUM',
	'GLX_BAD_HYPERPIPE_CONFIG_SGIX',
	'GLX_BAD_HYPERPIPE_SGIX',
	'GLX_BAD_SCREEN',
	'GLX_BAD_VALUE',
	'GLX_BAD_VISUAL',
	'GLX_BIND_TO_MIPMAP_TEXTURE_EXT',
	'GLX_BIND_TO_TEXTURE_RGBA_EXT',
	'GLX_BIND_TO_TEXTURE_RGB_EXT',
	'GLX_BIND_TO_TEXTURE_TARGETS_EXT',
	'GLX_BLENDED_RGBA_SGIS',
	'GLX_BLUE_SIZE',
	'GLX_BUFFER_CLOBBER_MASK_SGIX',
	'GLX_BUFFER_SIZE',
	'GLX_COLOR_INDEX_BIT',
	'GLX_COLOR_INDEX_BIT_SGIX',
	'GLX_COLOR_INDEX_TYPE',
	'GLX_COLOR_INDEX_TYPE_SGIX',
	'GLX_CONFIG_CAVEAT',
	'GLX_DAMAGED',
	'GLX_DAMAGED_SGIX',
	'GLX_DEPTH_BUFFER_BIT',
	'GLX_DEPTH_BUFFER_BIT_SGIX',
	'GLX_DEPTH_SIZE',
	'GLX_DIGITAL_MEDIA_PBUFFER_SGIX',
	'GLX_DIRECT_COLOR',
	'GLX_DIRECT_COLOR_EXT',
	'GLX_DONT_CARE',
	'GLX_DOUBLEBUFFER',
	'GLX_DRAWABLE_TYPE',
	'GLX_DRAWABLE_TYPE_SGIX',
	'GLX_EVENT_MASK',
	'GLX_EVENT_MASK_SGIX',
	'GLX_EXTENSIONS',
	'GLX_EXTENSION_NAME',
	'GLX_FBCONFIG_ID',
	'GLX_FBCONFIG_ID_SGIX',
	'GLX_FLOAT_COMPONENTS_NV',
	'GLX_FRONT_LEFT_BUFFER_BIT',
	'GLX_FRONT_LEFT_BUFFER_BIT_SGIX',
	'GLX_FRONT_LEFT_EXT',
	'GLX_FRONT_RIGHT_BUFFER_BIT',
	'GLX_FRONT_RIGHT_BUFFER_BIT_SGIX',
	'GLX_FRONT_RIGHT_EXT',
	'GLX_GLXEXT_VERSION',
	'GLX_GRAY_SCALE',
	'GLX_GRAY_SCALE_EXT',
	'GLX_GREEN_SIZE',
	'GLX_HEIGHT',
	'GLX_HEIGHT_SGIX',
	'GLX_HYPERPIPE_DISPLAY_PIPE_SGIX',
	'GLX_HYPERPIPE_ID_SGIX',
	'GLX_HYPERPIPE_PIPE_NAME_LENGTH_SGIX',
	'GLX_HYPERPIPE_PIXEL_AVERAGE_SGIX',
	'GLX_HYPERPIPE_RENDER_PIPE_SGIX',
	'GLX_HYPERPIPE_STEREO_SGIX',
	'GLX_LARGEST_PBUFFER',
	'GLX_LARGEST_PBUFFER_SGIX',
	'GLX_LEVEL',
	'GLX_MAX_PBUFFER_HEIGHT',
	'GLX_MAX_PBUFFER_HEIGHT_SGIX',
	'GLX_MAX_PBUFFER_PIXELS',
	'GLX_MAX_PBUFFER_PIXELS_SGIX',
	'GLX_MAX_PBUFFER_WIDTH',
	'GLX_MAX_PBUFFER_WIDTH_SGIX',
	'GLX_MIPMAP_TEXTURE_EXT',
	'GLX_MULTISAMPLE_SUB_RECT_HEIGHT_SGIS',
	'GLX_MULTISAMPLE_SUB_RECT_WIDTH_SGIS',
	'GLX_NONE',
	'GLX_NONE_EXT',
	'GLX_NON_CONFORMANT_CONFIG',
	'GLX_NON_CONFORMANT_VISUAL_EXT',
	'GLX_NO_EXTENSION',
	'GLX_OPTIMAL_PBUFFER_HEIGHT_SGIX',
	'GLX_OPTIMAL_PBUFFER_WIDTH_SGIX',
	'GLX_PBUFFER',
	'GLX_PBUFFER_BIT',
	'GLX_PBUFFER_BIT_SGIX',
	'GLX_PBUFFER_CLOBBER_MASK',
	'GLX_PBUFFER_HEIGHT',
	'GLX_PBUFFER_SGIX',
	'GLX_PBUFFER_WIDTH',
	'GLX_PIPE_RECT_LIMITS_SGIX',
	'GLX_PIPE_RECT_SGIX',
	'GLX_PIXMAP_BIT',
	'GLX_PIXMAP_BIT_SGIX',
	'GLX_PRESERVED_CONTENTS',
	'GLX_PRESERVED_CONTENTS_SGIX',
	'GLX_PSEUDO_COLOR',
	'GLX_PSEUDO_COLOR_EXT',
	'GLX_RED_SIZE',
	'GLX_RENDER_TYPE',
	'GLX_RENDER_TYPE_SGIX',
	'GLX_RGBA',
	'GLX_RGBA_BIT',
	'GLX_RGBA_BIT_SGIX',
	'GLX_RGBA_FLOAT_BIT_ARB',
	'GLX_RGBA_FLOAT_TYPE_ARB',
	'GLX_RGBA_TYPE',
	'GLX_RGBA_TYPE_SGIX',
	'GLX_SAMPLES',
	'GLX_SAMPLES_3DFX',
	'GLX_SAMPLES_ARB',
	'GLX_SAMPLES_SGIS',
	'GLX_SAMPLE_BUFFERS',
	'GLX_SAMPLE_BUFFERS_3DFX',
	'GLX_SAMPLE_BUFFERS_ARB',
	'GLX_SAMPLE_BUFFERS_BIT_SGIX',
	'GLX_SAMPLE_BUFFERS_SGIS',
	'GLX_SAVED',
	'GLX_SAVED_SGIX',
	'GLX_SCREEN',
	'GLX_SCREEN_EXT',
	'GLX_SHARE_CONTEXT_EXT',
	'GLX_SLOW_CONFIG',
	'GLX_SLOW_VISUAL_EXT',
	'GLX_STATIC_COLOR',
	'GLX_STATIC_COLOR_EXT',
	'GLX_STATIC_GRAY',
	'GLX_STATIC_GRAY_EXT',
	'GLX_STENCIL_BUFFER_BIT',
	'GLX_STENCIL_BUFFER_BIT_SGIX',
	'GLX_STENCIL_SIZE',
	'GLX_STEREO',
	'GLX_SWAP_COPY_OML',
	'GLX_SWAP_EXCHANGE_OML',
	'GLX_SWAP_METHOD_OML',
	'GLX_SWAP_UNDEFINED_OML',
	'GLX_SYNC_FRAME_SGIX',
	'GLX_SYNC_SWAP_SGIX',
	'GLX_TEXTURE_1D_BIT_EXT',
	'GLX_TEXTURE_1D_EXT',
	'GLX_TEXTURE_2D_BIT_EXT',
	'GLX_TEXTURE_2D_EXT',
	'GLX_TEXTURE_FORMAT_EXT',
	'GLX_TEXTURE_FORMAT_NONE_EXT',
	'GLX_TEXTURE_FORMAT_RGBA_EXT',
	'GLX_TEXTURE_FORMAT_RGB_EXT',
	'GLX_TEXTURE_RECTANGLE_BIT_EXT',
	'GLX_TEXTURE_RECTANGLE_EXT',
	'GLX_TEXTURE_TARGET_EXT',
	'GLX_TRANSPARENT_ALPHA_VALUE',
	'GLX_TRANSPARENT_ALPHA_VALUE_EXT',
	'GLX_TRANSPARENT_BLUE_VALUE',
	'GLX_TRANSPARENT_BLUE_VALUE_EXT',
	'GLX_TRANSPARENT_GREEN_VALUE',
	'GLX_TRANSPARENT_GREEN_VALUE_EXT',
	'GLX_TRANSPARENT_INDEX',
	'GLX_TRANSPARENT_INDEX_EXT',
	'GLX_TRANSPARENT_INDEX_VALUE',
	'GLX_TRANSPARENT_INDEX_VALUE_EXT',
	'GLX_TRANSPARENT_RED_VALUE',
	'GLX_TRANSPARENT_RED_VALUE_EXT',
	'GLX_TRANSPARENT_RGB',
	'GLX_TRANSPARENT_RGB_EXT',
	'GLX_TRANSPARENT_TYPE',
	'GLX_TRANSPARENT_TYPE_EXT',
	'GLX_TRUE_COLOR',
	'GLX_TRUE_COLOR_EXT',
	'GLX_USE_GL',
	'GLX_VENDOR',
	'GLX_VERSION',
	'GLX_VERSION_1_1',
	'GLX_VERSION_1_2',
	'GLX_VERSION_1_3',
	'GLX_VERSION_1_4',
	'GLX_VISUAL_CAVEAT_EXT',
	'GLX_VISUAL_ID',
	'GLX_VISUAL_ID_EXT',
	'GLX_VISUAL_SELECT_GROUP_SGIX',
	'GLX_WIDTH',
	'GLX_WIDTH_SGIX',
	'GLX_WINDOW',
	'GLX_WINDOW_BIT',
	'GLX_WINDOW_BIT_SGIX',
	'GLX_WINDOW_SGIX',
	'GLX_X_RENDERABLE',
	'GLX_X_RENDERABLE_SGIX',
	'GLX_X_VISUAL_TYPE',
	'GLX_X_VISUAL_TYPE_EXT',
	'GLX_Y_INVERTED_EXT',
	'GL_ALL_CLIENT_ATTRIB_BITS',
	'GL_ALPHA_BLEND_EQUATION_ATI',
	'GL_BYTE',
	'GL_DEBUG_ASSERT_MESA',
	'GL_DEBUG_OBJECT_MESA',
	'GL_DEBUG_PRINT_MESA',
	'GL_DEPTH_STENCIL_MESA',
	'GL_DOUBLE',
	'GL_FALSE',
	'GL_FLOAT',
	'GL_FRAGMENT_PROGRAM_CALLBACK_DATA_MESA',
	'GL_FRAGMENT_PROGRAM_CALLBACK_FUNC_MESA',
	'GL_FRAGMENT_PROGRAM_CALLBACK_MESA',
	'GL_FRAGMENT_PROGRAM_POSITION_MESA',
	'GL_INT',
	'GL_SHORT',
	'GL_TRACE_ALL_BITS_MESA',
	'GL_TRACE_ARRAYS_BIT_MESA',
	'GL_TRACE_ERRORS_BIT_MESA',
	'GL_TRACE_MASK_MESA',
	'GL_TRACE_NAME_MESA',
	'GL_TRACE_OPERATIONS_BIT_MESA',
	'GL_TRACE_PIXELS_BIT_MESA',
	'GL_TRACE_PRIMITIVES_BIT_MESA',
	'GL_TRACE_TEXTURES_BIT_MESA',
	'GL_TRUE',
	'GL_UNSIGNED_BYTE',
	'GL_UNSIGNED_INT',
	'GL_UNSIGNED_INT_24_8_MESA',
	'GL_UNSIGNED_INT_8_24_REV_MESA',
	'GL_UNSIGNED_SHORT',
	'GL_UNSIGNED_SHORT_15_1_MESA',
	'GL_UNSIGNED_SHORT_1_15_REV_MESA',
	'GL_VERTEX_PROGRAM_CALLBACK_DATA_MESA',
	'GL_VERTEX_PROGRAM_CALLBACK_FUNC_MESA',
	'GL_VERTEX_PROGRAM_CALLBACK_MESA',
	'GL_VERTEX_PROGRAM_POSITION_MESA',
	'XCNOENT',
	'XCNOMEM',
	'XCSUCCESS',
	'XK_0',
	'XK_1',
	'XK_2',
	'XK_3',
	'XK_4',
	'XK_5',
	'XK_6',
	'XK_7',
	'XK_8',
	'XK_9',
	'XK_A',
	'XK_AE',
	'XK_B',
	'XK_C',
	'XK_D',
	'XK_E',
	'XK_ENG',
	'XK_ETH',
	'XK_F',
	'XK_F1',
	'XK_F10',
	'XK_F11',
	'XK_F12',
	'XK_F13',
	'XK_F14',
	'XK_F15',
	'XK_F16',
	'XK_F17',
	'XK_F18',
	'XK_F19',
	'XK_F2',
	'XK_F20',
	'XK_F21',
	'XK_F22',
	'XK_F23',
	'XK_F24',
	'XK_F25',
	'XK_F26',
	'XK_F27',
	'XK_F28',
	'XK_F29',
	'XK_F3',
	'XK_F30',
	'XK_F31',
	'XK_F32',
	'XK_F33',
	'XK_F34',
	'XK_F35',
	'XK_F4',
	'XK_F5',
	'XK_F6',
	'XK_F7',
	'XK_F8',
	'XK_F9',
	'XK_G',
	'XK_H',
	'XK_I',
	'XK_J',
	'XK_K',
	'XK_KP_0',
	'XK_KP_1',
	'XK_KP_2',
	'XK_KP_3',
	'XK_KP_4',
	'XK_KP_5',
	'XK_KP_6',
	'XK_KP_7',
	'XK_KP_8',
	'XK_KP_9',
	'XK_KP_F1',
	'XK_KP_F2',
	'XK_KP_F3',
	'XK_KP_F4',
	'XK_L',
	'XK_L1',
	'XK_L10',
	'XK_L2',
	'XK_L3',
	'XK_L4',
	'XK_L5',
	'XK_L6',
	'XK_L7',
	'XK_L8',
	'XK_L9',
	'XK_M',
	'XK_N',
	'XK_O',
	'XK_OE',
	'XK_P',
	'XK_Q',
	'XK_R',
	'XK_R1',
	'XK_R10',
	'XK_R11',
	'XK_R12',
	'XK_R13',
	'XK_R14',
	'XK_R15',
	'XK_R2',
	'XK_R3',
	'XK_R4',
	'XK_R5',
	'XK_R6',
	'XK_R7',
	'XK_R8',
	'XK_R9',
	'XK_S',
	'XK_SCHWA',
	'XK_T',
	'XK_THORN',
	'XK_U',
	'XK_V',
	'XK_W',
	'XK_X',
	'XK_Y',
	'XK_Z',
	'X_HAVE_UTF8_STRING',
	'X_PROTOCOL',
	'X_PROTOCOL_REVISION',
	'_ATFILE_SOURCE',
	'_BITS_PTHREADTYPES_H',
	'_BITS_TYPESIZES_H',
	'_BITS_TYPES_H',
	'_BITS_WCHAR_H',
	'_BSD_SOURCE',
	'_ENDIAN_H',
	'_FEATURES_H',
	'_INTTYPES_H',
	'_ISOC99_SOURCE',
	'_LARGEFILE64_SOURCE',
	'_LARGEFILE_SOURCE',
	'_POSIX_C_SOURCE',
	'_POSIX_SOURCE',
	'_SIGSET_NWORDS',
	'_STDINT_H',
	'_STRUCT_TIMEVAL',
	'_SVID_SOURCE',
	'_SYS_CDEFS_H',
	'_SYS_SELECT_H',
	'_SYS_SYSMACROS_H',
	'_SYS_TYPES_H',
	'_XOPEN_SOURCE',
	'_XOPEN_SOURCE_EXTENDED',
	'__BIG_ENDIAN',
	'__BIT_TYPES_DEFINED__',
	'__FD_SETSIZE',
	'__GLIBC_HAVE_LONG_LONG',
	'__GLIBC_MINOR__',
	'__GLIBC__',
	'__GNU_LIBRARY__',
	'__LITTLE_ENDIAN',
	'__NFDBITS',
	'__PDP_ENDIAN',
	'__PTHREAD_MUTEX_HAVE_PREV',
	'__SIZEOF_PTHREAD_ATTR_T',
	'__SIZEOF_PTHREAD_BARRIERATTR_T',
	'__SIZEOF_PTHREAD_BARRIER_T',
	'__SIZEOF_PTHREAD_CONDATTR_T',
	'__SIZEOF_PTHREAD_COND_T',
	'__SIZEOF_PTHREAD_MUTEXATTR_T',
	'__SIZEOF_PTHREAD_MUTEX_T',
	'__SIZEOF_PTHREAD_RWLOCKATTR_T',
	'__SIZEOF_PTHREAD_RWLOCK_T',
	'__STDC_IEC_559_COMPLEX__',
	'__STDC_IEC_559__',
	'__STDC_ISO_10646__',
	'__USE_ANSI',
	'__USE_ATFILE',
	'__USE_BSD',
	'__USE_FORTIFY_LEVEL',
	'__USE_GNU',
	'__USE_ISOC99',
	'__USE_LARGEFILE',
	'__USE_LARGEFILE64',
	'__USE_MISC',
	'__USE_POSIX',
	'__USE_POSIX199309',
	'__USE_POSIX199506',
	'__USE_POSIX2',
	'__USE_SVID',
	'__USE_UNIX98',
	'__USE_XOPEN',
	'__USE_XOPEN2K',
	'__USE_XOPEN_EXTENDED',
	'__WCHAR_MAX',
	'__WCHAR_MIN',
	'__WORDSIZE',
	'__WORDSIZE_COMPAT32'
]




"""Passing of None as an array data-type
"""
REGISTRY_NAME = 'nones'
from OpenGL.arrays import formathandler

class NoneHandler( formathandler.FormatHandler ):
	"""Numpy-specific data-type handler for OpenGL"""
	HANDLED_TYPES = (type(None), )
	def from_param( self, value ):
		"""Convert to a ctypes pointer value"""
		return None
	def dataPointer( self, value ):
		"""return long for pointer value"""
		return None
	def voidDataPointer( cls, value ):
		"""Given value in a known data-pointer type, return void_p for pointer"""
		return None
	def asArray( self, value, typeCode=None ):
		"""Given a value, convert to array representation"""
		return None
	def arrayToGLType( self, value ):
		"""Given a value, guess OpenGL type of the corresponding pointer"""
		raise TypeError( """Can't guess type of a NULL pointer""" )
	def arraySize( self, value, typeCode = None ):
		"""Given a data-value, calculate dimensions for the array"""
		return 0
	def arrayByteCount( self, value, typeCode = None ):
		"""Given a data-value, calculate number of bytes required to represent"""
		return 0
	def zeros( self, shape, typeCode= None ):
		"""Create an array of given shape with given typeCode"""
		raise TypeError( """Can't create NULL pointer filled with values""" )
	def ones( self, shape, typeCode= None ):
		"""Create an array of given shape with given typeCode"""
		raise TypeError( """Can't create NULL pointer filled with values""" )
	def unitSize( self, value, typeCode=None ):
		"""Determine unit size of an array (if possible)"""
		raise TypeError( """Can't determine unit size of a null pointer""" )
	def dimensions( self, value, typeCode=None ):
		"""Determine dimensions of the passed array value (if possible)"""
		return (0,)



'''OpenGL extension EXT.misc_attribute

Overview (from the spec)
	
	EXT_misc_attribute extends the list of attribute groups. It provides
	a miscellaneous group, controlled by the MISC_BIT_EXT bit, that contains
	the attribute state of extensions that don't logically fit in any other
	group. 

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/misc_attribute.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_misc_attribute'



def glInitMiscAttributeEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension NV.gpu_program4

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/NV/gpu_program4.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_gpu_program4'
GL_MIN_PROGRAM_TEXEL_OFFSET_NV = constant.Constant( 'GL_MIN_PROGRAM_TEXEL_OFFSET_NV', 0x8904 )
GL_MAX_PROGRAM_TEXEL_OFFSET_NV = constant.Constant( 'GL_MAX_PROGRAM_TEXEL_OFFSET_NV', 0x8905 )
GL_PROGRAM_ATTRIB_COMPONENTS_NV = constant.Constant( 'GL_PROGRAM_ATTRIB_COMPONENTS_NV', 0x8906 )
GL_PROGRAM_RESULT_COMPONENTS_NV = constant.Constant( 'GL_PROGRAM_RESULT_COMPONENTS_NV', 0x8907 )
GL_MAX_PROGRAM_ATTRIB_COMPONENTS_NV = constant.Constant( 'GL_MAX_PROGRAM_ATTRIB_COMPONENTS_NV', 0x8908 )
GL_MAX_PROGRAM_RESULT_COMPONENTS_NV = constant.Constant( 'GL_MAX_PROGRAM_RESULT_COMPONENTS_NV', 0x8909 )
GL_MAX_PROGRAM_GENERIC_ATTRIBS_NV = constant.Constant( 'GL_MAX_PROGRAM_GENERIC_ATTRIBS_NV', 0x8DA5 )
GL_MAX_PROGRAM_GENERIC_RESULTS_NV = constant.Constant( 'GL_MAX_PROGRAM_GENERIC_RESULTS_NV', 0x8DA6 )
glProgramLocalParameterI4iNV = platform.createExtensionFunction( 
	'glProgramLocalParameterI4iNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLint, constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glProgramLocalParameterI4iNV( GLenum(target), GLuint(index), GLint(x), GLint(y), GLint(z), GLint(w) ) -> None',
	argNames = ('target', 'index', 'x', 'y', 'z', 'w',),
)

glProgramLocalParameterI4ivNV = platform.createExtensionFunction( 
	'glProgramLocalParameterI4ivNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, arrays.GLintArray,),
	doc = 'glProgramLocalParameterI4ivNV( GLenum(target), GLuint(index), GLintArray(params) ) -> None',
	argNames = ('target', 'index', 'params',),
)

glProgramLocalParametersI4ivNV = platform.createExtensionFunction( 
	'glProgramLocalParametersI4ivNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLsizei, arrays.GLintArray,),
	doc = 'glProgramLocalParametersI4ivNV( GLenum(target), GLuint(index), GLsizei(count), GLintArray(params) ) -> None',
	argNames = ('target', 'index', 'count', 'params',),
)

glProgramLocalParameterI4uiNV = platform.createExtensionFunction( 
	'glProgramLocalParameterI4uiNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLuint, constants.GLuint, constants.GLuint, constants.GLuint,),
	doc = 'glProgramLocalParameterI4uiNV( GLenum(target), GLuint(index), GLuint(x), GLuint(y), GLuint(z), GLuint(w) ) -> None',
	argNames = ('target', 'index', 'x', 'y', 'z', 'w',),
)

glProgramLocalParameterI4uivNV = platform.createExtensionFunction( 
	'glProgramLocalParameterI4uivNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, arrays.GLuintArray,),
	doc = 'glProgramLocalParameterI4uivNV( GLenum(target), GLuint(index), GLuintArray(params) ) -> None',
	argNames = ('target', 'index', 'params',),
)

glProgramLocalParametersI4uivNV = platform.createExtensionFunction( 
	'glProgramLocalParametersI4uivNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLsizei, arrays.GLuintArray,),
	doc = 'glProgramLocalParametersI4uivNV( GLenum(target), GLuint(index), GLsizei(count), GLuintArray(params) ) -> None',
	argNames = ('target', 'index', 'count', 'params',),
)

glProgramEnvParameterI4iNV = platform.createExtensionFunction( 
	'glProgramEnvParameterI4iNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLint, constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glProgramEnvParameterI4iNV( GLenum(target), GLuint(index), GLint(x), GLint(y), GLint(z), GLint(w) ) -> None',
	argNames = ('target', 'index', 'x', 'y', 'z', 'w',),
)

glProgramEnvParameterI4ivNV = platform.createExtensionFunction( 
	'glProgramEnvParameterI4ivNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, arrays.GLintArray,),
	doc = 'glProgramEnvParameterI4ivNV( GLenum(target), GLuint(index), GLintArray(params) ) -> None',
	argNames = ('target', 'index', 'params',),
)

glProgramEnvParametersI4ivNV = platform.createExtensionFunction( 
	'glProgramEnvParametersI4ivNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLsizei, arrays.GLintArray,),
	doc = 'glProgramEnvParametersI4ivNV( GLenum(target), GLuint(index), GLsizei(count), GLintArray(params) ) -> None',
	argNames = ('target', 'index', 'count', 'params',),
)

glProgramEnvParameterI4uiNV = platform.createExtensionFunction( 
	'glProgramEnvParameterI4uiNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLuint, constants.GLuint, constants.GLuint, constants.GLuint,),
	doc = 'glProgramEnvParameterI4uiNV( GLenum(target), GLuint(index), GLuint(x), GLuint(y), GLuint(z), GLuint(w) ) -> None',
	argNames = ('target', 'index', 'x', 'y', 'z', 'w',),
)

glProgramEnvParameterI4uivNV = platform.createExtensionFunction( 
	'glProgramEnvParameterI4uivNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, arrays.GLuintArray,),
	doc = 'glProgramEnvParameterI4uivNV( GLenum(target), GLuint(index), GLuintArray(params) ) -> None',
	argNames = ('target', 'index', 'params',),
)

glProgramEnvParametersI4uivNV = platform.createExtensionFunction( 
	'glProgramEnvParametersI4uivNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLsizei, arrays.GLuintArray,),
	doc = 'glProgramEnvParametersI4uivNV( GLenum(target), GLuint(index), GLsizei(count), GLuintArray(params) ) -> None',
	argNames = ('target', 'index', 'count', 'params',),
)

glGetProgramLocalParameterIivNV = platform.createExtensionFunction( 
	'glGetProgramLocalParameterIivNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, arrays.GLintArray,),
	doc = 'glGetProgramLocalParameterIivNV( GLenum(target), GLuint(index), GLintArray(params) ) -> None',
	argNames = ('target', 'index', 'params',),
)

glGetProgramLocalParameterIuivNV = platform.createExtensionFunction( 
	'glGetProgramLocalParameterIuivNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, arrays.GLuintArray,),
	doc = 'glGetProgramLocalParameterIuivNV( GLenum(target), GLuint(index), GLuintArray(params) ) -> None',
	argNames = ('target', 'index', 'params',),
)

glGetProgramEnvParameterIivNV = platform.createExtensionFunction( 
	'glGetProgramEnvParameterIivNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, arrays.GLintArray,),
	doc = 'glGetProgramEnvParameterIivNV( GLenum(target), GLuint(index), GLintArray(params) ) -> None',
	argNames = ('target', 'index', 'params',),
)

glGetProgramEnvParameterIuivNV = platform.createExtensionFunction( 
	'glGetProgramEnvParameterIuivNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, arrays.GLuintArray,),
	doc = 'glGetProgramEnvParameterIuivNV( GLenum(target), GLuint(index), GLuintArray(params) ) -> None',
	argNames = ('target', 'index', 'params',),
)


def glInitGpuProgram4NV():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension APPLE.element_array

This module customises the behaviour of the 
OpenGL.raw.GL.APPLE.element_array to provide a more 
Python-friendly API
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.APPLE.element_array import *
### END AUTOGENERATED SECTION
from OpenGL.GL import glget
glget.addGLGetConstant( GL_ELEMENT_ARRAY_TYPE_APPLE, (1,) ) # check size...



'''OpenGL extension ATI.texture_float

Overview (from the spec)
	
	This extension adds texture internal formats with 32 and 16 bit
	floating-point components.  The 32 bit floating-point components
	are in the standard IEEE float format.  The 16 bit floating-point
	components have 1 sign bit, 5 exponent bits, and 10 mantissa bits.
	Floating-point components are clamped to the limits of the range
	representable by their format.
	

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ATI/texture_float.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ATI_texture_float'
GL_RGBA_FLOAT32_ATI = constant.Constant( 'GL_RGBA_FLOAT32_ATI', 0x8814 )
GL_RGB_FLOAT32_ATI = constant.Constant( 'GL_RGB_FLOAT32_ATI', 0x8815 )
GL_ALPHA_FLOAT32_ATI = constant.Constant( 'GL_ALPHA_FLOAT32_ATI', 0x8816 )
GL_INTENSITY_FLOAT32_ATI = constant.Constant( 'GL_INTENSITY_FLOAT32_ATI', 0x8817 )
GL_LUMINANCE_FLOAT32_ATI = constant.Constant( 'GL_LUMINANCE_FLOAT32_ATI', 0x8818 )
GL_LUMINANCE_ALPHA_FLOAT32_ATI = constant.Constant( 'GL_LUMINANCE_ALPHA_FLOAT32_ATI', 0x8819 )
GL_RGBA_FLOAT16_ATI = constant.Constant( 'GL_RGBA_FLOAT16_ATI', 0x881A )
GL_RGB_FLOAT16_ATI = constant.Constant( 'GL_RGB_FLOAT16_ATI', 0x881B )
GL_ALPHA_FLOAT16_ATI = constant.Constant( 'GL_ALPHA_FLOAT16_ATI', 0x881C )
GL_INTENSITY_FLOAT16_ATI = constant.Constant( 'GL_INTENSITY_FLOAT16_ATI', 0x881D )
GL_LUMINANCE_FLOAT16_ATI = constant.Constant( 'GL_LUMINANCE_FLOAT16_ATI', 0x881E )
GL_LUMINANCE_ALPHA_FLOAT16_ATI = constant.Constant( 'GL_LUMINANCE_ALPHA_FLOAT16_ATI', 0x881F )


def glInitTextureFloatATI():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



#class LazyWrapper( object ):
#	"""Object which holds a lazy-loading function and a wrapper object
#	
#	Applies the wrapper to the baseFunction when the baseFunction is 
#	defined, otherwise just calls the base function.
#	"""
#	def __init__( self, baseFunction, wrapperFunction ):
#		"""Initialize the lazy wrapper 
#		
#		baseFunction -- the base platform function object 
#		wrapperFunction -- the wrapper function, must take 
#			the baseFunction as its first argument, then the arguments
#			for the Pythonic API for the function
#		"""
#		self.baseFunction = baseFunction
#		self.wrapperFunction = wrapperFunction
#	def __call__( self, *args, **named ):
#		if self.baseFunction:
#			return self.wrapperFunction( self.baseFunction, *args, **named )
#		else:
#			return self.baseFunction( *args, **named )
#	def __nonzero__( self ):
#		return bool( self.baseFunction )

def lazy( baseFunction ):
	"""Produce a lazy-binding decorator that uses baseFunction
	"""
	def wrap( wrapper ):
		"""Wrap wrapper with baseFunction"""
		def __call__( self, *args, **named ):
			if baseFunction:
				return wrapper( baseFunction, *args, **named )
			else:
				return baseFunction( *args, **named )
		def __nonzero__( self ):
			return bool( baseFunction )
		_with_wrapper = type( wrapper.__name__, (object,), {
			'__call__': __call__,
			'__doc__': wrapper.__doc__,
			'__nonzero__': __nonzero__,
			'restype': getattr(wrapper, 'restype',getattr(baseFunction,'restype',None)),
		} )
		with_wrapper = _with_wrapper()
		with_wrapper.__name__ = wrapper.__name__
		with_wrapper.baseFunction = baseFunction 
		with_wrapper.wrapperFunction = wrapper
		return with_wrapper
	return wrap 


if __name__ == "__main__":
	from OpenGL.raw import GLU
	func = GLU.gluNurbsCallbackData
	output = []
	def testwrap( base ):
		"Testing"
		output.append( base )
	testlazy = lazy( func )( testwrap )
	testlazy( )
	assert testlazy.__doc__ == "Testing" 
	assert testlazy.__class__.__name__ == 'testwrap'
	assert testlazy.__name__ == 'testwrap'
	assert testlazy.baseFunction is func 
	assert testlazy.wrapperFunction is testwrap
	assert output 



'''OpenGL extension IBM.rasterpos_clip

Overview (from the spec)
	
	IBM_rasterpos_clip extends the semantics of the RasterPos functions.  It
	provides an enable that allows a raster position that would normally be
	clipped to be treated as a valid (albeit out-of-viewport) position.
	
	This extension allows applications to specify geometry-aligned pixel
	primitives that may be partially off-screen.  These primitives are
	tested on a pixel-by-pixel basis without being rejected completely
	because of an invalid raster position.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/IBM/rasterpos_clip.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_IBM_rasterpos_clip'
GL_RASTER_POSITION_UNCLIPPED_IBM = constant.Constant( 'GL_RASTER_POSITION_UNCLIPPED_IBM', 0x19262 )
glget.addGLGetConstant( GL_RASTER_POSITION_UNCLIPPED_IBM, (1,) )


def glInitRasterposClipIBM():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.geometry_shader4

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/geometry_shader4.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_geometry_shader4'
GL_GEOMETRY_SHADER_EXT = constant.Constant( 'GL_GEOMETRY_SHADER_EXT', 0x8DD9 )
GL_MAX_GEOMETRY_VARYING_COMPONENTS_EXT = constant.Constant( 'GL_MAX_GEOMETRY_VARYING_COMPONENTS_EXT', 0x8DDD )
GL_MAX_VERTEX_VARYING_COMPONENTS_EXT = constant.Constant( 'GL_MAX_VERTEX_VARYING_COMPONENTS_EXT', 0x8DDE )
GL_MAX_VARYING_COMPONENTS_EXT = constant.Constant( 'GL_MAX_VARYING_COMPONENTS_EXT', 0x8B4B )
GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_EXT = constant.Constant( 'GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_EXT', 0x8DDF )
GL_MAX_GEOMETRY_OUTPUT_VERTICES_EXT = constant.Constant( 'GL_MAX_GEOMETRY_OUTPUT_VERTICES_EXT', 0x8DE0 )
GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_EXT = constant.Constant( 'GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_EXT', 0x8DE1 )
glProgramParameteriEXT = platform.createExtensionFunction( 
	'glProgramParameteriEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, constants.GLint,),
	doc = 'glProgramParameteriEXT( GLuint(program), GLenum(pname), GLint(value) ) -> None',
	argNames = ('program', 'pname', 'value',),
)


def glInitGeometryShader4EXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension NV.texture_compression_vtc

Overview (from the spec)
	
	This extension adds support for the VTC 3D texture compression
	formats, which are analogous to the S3TC texture compression formats,
	with the addition of some retiling in the Z direction.  VTC has the
	same compression ratio as S3TC and uses 4x4x1, 4x4x2, or 4x4x4
	blocks.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/NV/texture_compression_vtc.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_texture_compression_vtc'



def glInitTextureCompressionVtcNV():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.blend_subtract

Overview (from the spec)
	
	Two additional blending equations are specified using the interface
	defined by EXT_blend_minmax.  These equations are similar to the
	default blending equation, but produce the difference of its left
	and right hand sides, rather than the sum.  Image differences are
	useful in many image processing applications.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/blend_subtract.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_blend_subtract'
GL_FUNC_SUBTRACT_EXT = constant.Constant( 'GL_FUNC_SUBTRACT_EXT', 0x800A )
GL_FUNC_REVERSE_SUBTRACT_EXT = constant.Constant( 'GL_FUNC_REVERSE_SUBTRACT_EXT', 0x800B )


def glInitBlendSubtractEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.geometry_shader4

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ARB/geometry_shader4.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_geometry_shader4'
GL_LINES_ADJACENCY_ARB = constant.Constant( 'GL_LINES_ADJACENCY_ARB', 0xA )
GL_LINE_STRIP_ADJACENCY_ARB = constant.Constant( 'GL_LINE_STRIP_ADJACENCY_ARB', 0xB )
GL_TRIANGLES_ADJACENCY_ARB = constant.Constant( 'GL_TRIANGLES_ADJACENCY_ARB', 0xC )
GL_TRIANGLE_STRIP_ADJACENCY_ARB = constant.Constant( 'GL_TRIANGLE_STRIP_ADJACENCY_ARB', 0xD )
GL_PROGRAM_POINT_SIZE_ARB = constant.Constant( 'GL_PROGRAM_POINT_SIZE_ARB', 0x8642 )
GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_ARB = constant.Constant( 'GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_ARB', 0x8C29 )
GL_FRAMEBUFFER_ATTACHMENT_LAYERED_ARB = constant.Constant( 'GL_FRAMEBUFFER_ATTACHMENT_LAYERED_ARB', 0x8DA7 )
GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_ARB = constant.Constant( 'GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_ARB', 0x8DA8 )
GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_ARB = constant.Constant( 'GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_ARB', 0x8DA9 )
GL_GEOMETRY_SHADER_ARB = constant.Constant( 'GL_GEOMETRY_SHADER_ARB', 0x8DD9 )
GL_GEOMETRY_VERTICES_OUT_ARB = constant.Constant( 'GL_GEOMETRY_VERTICES_OUT_ARB', 0x8DDA )
GL_GEOMETRY_INPUT_TYPE_ARB = constant.Constant( 'GL_GEOMETRY_INPUT_TYPE_ARB', 0x8DDB )
GL_GEOMETRY_OUTPUT_TYPE_ARB = constant.Constant( 'GL_GEOMETRY_OUTPUT_TYPE_ARB', 0x8DDC )
GL_MAX_GEOMETRY_VARYING_COMPONENTS_ARB = constant.Constant( 'GL_MAX_GEOMETRY_VARYING_COMPONENTS_ARB', 0x8DDD )
GL_MAX_VERTEX_VARYING_COMPONENTS_ARB = constant.Constant( 'GL_MAX_VERTEX_VARYING_COMPONENTS_ARB', 0x8DDE )
GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_ARB = constant.Constant( 'GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_ARB', 0x8DDF )
GL_MAX_GEOMETRY_OUTPUT_VERTICES_ARB = constant.Constant( 'GL_MAX_GEOMETRY_OUTPUT_VERTICES_ARB', 0x8DE0 )
GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_ARB = constant.Constant( 'GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_ARB', 0x8DE1 )
glProgramParameteriARB = platform.createExtensionFunction( 
	'glProgramParameteriARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, constants.GLint,),
	doc = 'glProgramParameteriARB( GLuint(program), GLenum(pname), GLint(value) ) -> None',
	argNames = ('program', 'pname', 'value',),
)

glFramebufferTextureARB = platform.createExtensionFunction( 
	'glFramebufferTextureARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, constants.GLuint, constants.GLint,),
	doc = 'glFramebufferTextureARB( GLenum(target), GLenum(attachment), GLuint(texture), GLint(level) ) -> None',
	argNames = ('target', 'attachment', 'texture', 'level',),
)

glFramebufferTextureLayerARB = platform.createExtensionFunction( 
	'glFramebufferTextureLayerARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, constants.GLuint, constants.GLint, constants.GLint,),
	doc = 'glFramebufferTextureLayerARB( GLenum(target), GLenum(attachment), GLuint(texture), GLint(level), GLint(layer) ) -> None',
	argNames = ('target', 'attachment', 'texture', 'level', 'layer',),
)

glFramebufferTextureFaceARB = platform.createExtensionFunction( 
	'glFramebufferTextureFaceARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, constants.GLuint, constants.GLint, constants.GLenum,),
	doc = 'glFramebufferTextureFaceARB( GLenum(target), GLenum(attachment), GLuint(texture), GLint(level), GLenum(face) ) -> None',
	argNames = ('target', 'attachment', 'texture', 'level', 'face',),
)


def glInitGeometryShader4ARB():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )
